@@ -1,7 +1,7 @@
 /* ay.h: AY-8-3912 routines
    Copyright (c) 1999-2004 Philip Kendall
 
-   $Id: ay.h,v 1.15 2004/06/15 09:48:40 pak21 Exp $
+   $Id: ay.h 2993 2007-06-17 13:54:49Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -36,12 +35,11 @@ typedef struct ayinfo {
   libspectrum_byte registers[ AY_REGISTERS ];
 } ayinfo;
 
+int ay_init( void );
+
 libspectrum_byte ay_registerport_read( libspectrum_word port, int *attached );
 void ay_registerport_write( libspectrum_word port, libspectrum_byte b );
 
 void ay_dataport_write( libspectrum_word port, libspectrum_byte b );
 
-int ay_from_snapshot( libspectrum_snap *snap, int capabilities );
-int ay_to_snapshot( libspectrum_snap *snap );
-
 #endif			/* #ifndef FUSE_AY_H */
