@@ -1,7 +1,7 @@
 /* command.c: Parse a debugger command
-   Copyright (c) 2002 Philip Kendall
+   Copyright (c) 2002-2008 Philip Kendall
 
-   $Id: command.c,v 1.16 2003/12/02 14:56:32 pak21 Exp $
+   $Id: command.c 3631 2008-05-26 12:22:29Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -32,6 +31,7 @@
 
 #include "debugger.h"
 #include "debugger_internals.h"
+#include "mempool.h"
 #include "ui/ui.h"
 #include "z80/z80.h"
 #include "z80/z80_macros.h"
@@ -49,6 +49,8 @@ int yywrap( void );
 int
 debugger_command_evaluate( const char *command )
 {
+  if( !command ) return 0;
+
   if( command_buffer ) free( command_buffer );
 
   command_buffer = strdup( command );
@@ -63,6 +65,9 @@ debugger_command_evaluate( const char *command )
   /* Parse the command */
   yyparse();
 
+  /* And free any memory we allocated while parsing */
+  mempool_free( debugger_memory_pool );
+
   ui_debugger_update();
 
   return 0;
