@@ -1,7 +1,7 @@
-/* disassemble.h: Fuse's disassembler
+/* disassemble.c: Fuse's disassembler
    Copyright (c) 2002-2003 Darren Salt, Philip Kendall
 
-   $Id: disassemble.c,v 1.10 2003/11/14 11:46:47 pak21 Exp $
+   $Id: disassemble.c 3115 2007-08-19 02:49:14Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,16 +13,15 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
    Darren: linux@youmustbejoking.demon.co.uk
 
-   Philip: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   Philip: philip-fuse@shadowmagic.org.uk
 
 */
 
