@@ -1,7 +1,7 @@
 /* display.h: Routines for printing the Spectrum's screen
-   Copyright (c) 1999-2003 Philip Kendall
+   Copyright (c) 1999-2006 Philip Kendall
 
-   $Id: display.h,v 1.21 2003/07/11 11:03:25 pak21 Exp $
+   $Id: display.h 2889 2007-05-26 17:45:08Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -50,7 +49,7 @@
 #define DISPLAY_BORDER_WIDTH  ( DISPLAY_BORDER_WIDTH_COLS * 16 )
 /* Aspect corrected border width */
 #define DISPLAY_BORDER_ASPECT_WIDTH  ( DISPLAY_BORDER_WIDTH_COLS * 8 )
-/* Each main screen column can produce only 8 pixels in any mode */
+/* Each main screen row can produce only 8 pixels in any mode */
 #define DISPLAY_BORDER_HEIGHT ( DISPLAY_BORDER_HEIGHT_COLS * 8 )
 
 /* The width and height of the window we'll be displaying */
@@ -65,10 +64,6 @@
 
 extern int display_ui_initialised;
 
-extern libspectrum_word
-  display_image[ 2 * DISPLAY_SCREEN_HEIGHT ][ DISPLAY_SCREEN_WIDTH ];
-extern ptrdiff_t display_pitch;
-
 extern libspectrum_byte display_lores_border;
 extern libspectrum_byte display_hires_border;
 
@@ -80,12 +75,7 @@ extern libspectrum_word display_attr_start[ DISPLAY_HEIGHT ];
 int display_init(int *argc, char ***argv);
 void display_line(void);
 
-void display_dirty( libspectrum_word address );
-void display_putpixel( int x, int y, int colour );
-void display_plot8( int x, int y, libspectrum_byte data, libspectrum_byte ink,
-		    libspectrum_byte paper );
-void display_plot16( int x, int y, libspectrum_word data, libspectrum_byte ink,
-		     libspectrum_byte paper);
+void display_dirty( libspectrum_word offset );
 
 void display_parse_attr( libspectrum_byte attr, libspectrum_byte *ink,
 			 libspectrum_byte *paper );
@@ -95,8 +85,14 @@ void display_set_hires_border(int colour);
 int display_dirty_border(void);
 
 int display_frame(void);
+void display_refresh_main_screen(void);
 void display_refresh_all(void);
 
-libspectrum_word display_get_addr( int x, int y );
+#define display_get_addr( x, y ) \
+  scld_last_dec.name.altdfile ? display_line_start[(y)]+(x)+ALTDFILE_OFFSET : \
+  display_line_start[(y)]+(x)
+int display_getpixel( int x, int y );
+
+void display_update_critical( int x, int y );
 
 #endif			/* #ifndef FUSE_DISPLAY_H */
