@@ -1,7 +1,7 @@
 /* fuse.c: The Free Unix Spectrum Emulator
-   Copyright (c) 1999-2004 Philip Kendall
+   Copyright (c) 1999-2009 Philip Kendall
 
-   $Id: fuse.c,v 1.120 2004/05/31 17:38:19 pak21 Exp $
+   $Id: fuse.c 3942 2009-01-10 14:18:46Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -29,14 +28,14 @@
 #include <errno.h>
 #include <limits.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <sys/types.h>
+#include <time.h>
 
-#ifndef WIN32
-#include <sys/utsname.h>
-#else				/* #ifndef WIN32 */
+#ifdef WIN32
 #include <windows.h>
-#endif				/* #ifndef WIN32 */
+#endif				/* #ifdef WIN32 */
 
 #include <unistd.h>
 
@@ -44,36 +43,46 @@
 #include <SDL.h>		/* Needed on MacOS X and Windows */
 #endif				/* #ifdef UI_SDL */
 
+#include "ay.h"
 #include "dck.h"
 #include "debugger/debugger.h"
+#include "disk/beta.h"
+#include "disk/fdd.h"
 #include "display.h"
+#include "divide.h"
 #include "event.h"
 #include "fuse.h"
+#include "if1.h"
+#include "if2.h"
 #include "joystick.h"
 #include "keyboard.h"
+#include "kempmouse.h"
 #include "machine.h"
 #include "memory.h"
 #include "pokefinder/pokefinder.h"
 #include "printer.h"
+#include "profile.h"
 #include "psg.h"
 #include "rzx.h"
+#include "scld.h"
 #include "settings.h"
 #include "simpleide.h"
+#include "slt.h"
 #include "snapshot.h"
 #include "sound.h"
-#include "sound/lowlevel.h"
 #include "spectrum.h"
 #include "tape.h"
-#include "timer.h"
-#include "trdos.h"
+#include "timer/timer.h"
 #include "ui/ui.h"
 #include "ui/scaler/scaler.h"
+#include "ula.h"
+#include "unittests/unittests.h"
 #include "utils.h"
 #include "zxatasp.h"
 #include "zxcf.h"
 
 #ifdef USE_WIDGET
-#include "widget/widget.h"
+#include "ui/widget/widget.h"
 #endif                          /* #ifdef USE_WIDGET */
 
 #include "z80/z80.h"
@@ -90,23 +99,18 @@ int fuse_exiting;
 /* Is Spectrum emulation currently paused, and if so, how many times? */
 int fuse_emulation_paused;
 
-/* Are we going to try and use the sound card; this differs from
-   sound.c:sound_enabled in that this gives a desire, whereas sound_enabled
-   is an actual state; when the Spectrum emulation is not running, this
-   stores whether we try to reenable the sound card afterwards */
-int fuse_sound_in_use;
-
 /* The creator information we'll store in file formats that support this */
 libspectrum_creator *fuse_creator;
 
 /* The earliest version of libspectrum we need */
-static const char *LIBSPECTRUM_MIN_VERSION = "0.2.0.1";
+static const char *LIBSPECTRUM_MIN_VERSION = "0.5.0";
 
 /* The various types of file we may want to run on startup */
 typedef struct start_files_t {
 
   const char *disk_plus3;
-  const char *disk_trdos;
+  const char *disk_plusd;
+  const char *disk_beta;
   const char *dock;
   const char *if2;
   const char *playback;
@@ -117,6 +121,8 @@ typedef struct start_files_t {
   const char *simpleide_master, *simpleide_slave;
   const char *zxatasp_master, *zxatasp_slave;
   const char *zxcf;
+  const char *divide_master, *divide_slave;
+  const char *mdr[8];
 
 } start_files_t;
 
@@ -134,33 +140,54 @@ static int do_start_files( start_files_t *start_files );
 
 static int fuse_end(void);
 
+#ifdef UI_WIN32
+int fuse_main(int argc, char **argv)
+#else
 int main(int argc, char **argv)
+#endif
 {
+  int r;
+
+#ifdef WIN32
+  SetErrorMode( SEM_FAILCRITICALERRORS | SEM_NOOPENFILEERRORBOX );
+#endif
+
   if(fuse_init(argc,argv)) {
-    fprintf(stderr,"%s: error initalising -- giving up!\n", fuse_progname);
+    fprintf(stderr,"%s: error initialising -- giving up!\n", fuse_progname);
     return 1;
   }
 
   if( settings_current.show_help ||
       settings_current.show_version ) return 0;
 
-  while( !fuse_exiting ) {
-    z80_do_opcodes();
-    event_do_events();
+  if( settings_current.unittests ) {
+    r = unittests_run();
+  } else {
+    while( !fuse_exiting ) {
+      z80_do_opcodes();
+      event_do_events();
+    }
+    r = 0;
   }
 
   fuse_end();
   
-  return 0;
+  return r;
 
 }
 
+#include "mempool.h"
+
 static int fuse_init(int argc, char **argv)
 {
   int error, first_arg;
   char *start_scaler;
   start_files_t start_files;
 
+  /* Seed the bad but widely-available random number
+     generator with the current time */
+  srand( (unsigned)time( NULL ) );
+
   fuse_progname=argv[0];
   libspectrum_error_function = ui_libspectrum_error;
 
@@ -169,7 +196,7 @@ static int fuse_init(int argc, char **argv)
 	      strerror( errno ) );
     return 1;
   }
-  strcat( fuse_directory, "/" );
+  strcat( fuse_directory, FUSE_DIR_SEP_STR );
 
   if( settings_init( &first_arg, argc, argv ) ) return 1;
 
@@ -183,11 +210,15 @@ static int fuse_init(int argc, char **argv)
 
   start_scaler = strdup( settings_current.start_scaler_mode );
   if( !start_scaler ) {
-    ui_error( UI_ERROR_ERROR, "Out of memory at %s: %d", __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
+  /* Windows will create a console for our output if there isn't one already,
+   * so we don't display the copyright message on Win32. */
+#ifndef WIN32
   fuse_show_copyright();
+#endif
 
   /* FIXME: order of these initialisation calls. Work out what depends on
      what */
@@ -222,39 +253,36 @@ static int fuse_init(int argc, char **argv)
   if( !geteuid() ) { setuid( getuid() ); }
 #endif				/* #ifdef HAVE_GETEUID */
 
+  if( mempool_init() ) return 1;
   if( memory_init() ) return 1;
 
   if( debugger_init() ) return 1;
 
+  if( spectrum_init() ) return 1;
   if( printer_init() ) return 1;
   if( rzx_init() ) return 1;
   if( psg_init() ) return 1;
-  if( trdos_init() ) return 1;
+  if( beta_init() ) return 1;
+  if( plusd_init() ) return 1;
+  if( fdd_init_events() ) return 1;
   if( simpleide_init() ) return 1;
   if( zxatasp_init() ) return 1;
   if( zxcf_init() ) return 1;
+  if( if1_init() ) return 1;
+  if( if2_init() ) return 1;
+  if( divide_init() ) return 1;
+  if( scld_init() ) return 1;
+  if( ula_init() ) return 1;
+  if( ay_init() ) return 1;
+  if( slt_init() ) return 1;
+  if( profile_init() ) return 1;
+  if( kempmouse_init() ) return 1;
 
   error = pokefinder_clear(); if( error ) return error;
 
-  z80_init();
+  if( z80_init() ) return 1;
 
-  fuse_sound_in_use = 0;
-  if( settings_current.sound && settings_current.emulation_speed == 100 )
-    sound_init( settings_current.sound_device );
-
-  /* Timing: if we're not producing sound or using sound routines which
-     don't do timing control for us, use the timer_* routines */
-#ifdef ALWAYS_USE_TIMER
-  if( sound_enabled ) fuse_sound_in_use = 1;
   if( timer_init() ) return 1;
-#else			/* #ifdef ALWAYS_USE_TIMER */
-  if(sound_enabled) {
-    fuse_sound_in_use = 1;
-  } else {
-    settings_current.sound = 0;
-    if(timer_init()) return 1;
-  }
-#endif			/* #ifdef ALWAYS_USE_TIMER */
 
   error = timer_estimate_reset(); if( error ) return error;
 
@@ -273,10 +301,14 @@ static int fuse_init(int argc, char **argv)
   if( parse_nonoption_args( argc, argv, first_arg, &start_files ) ) return 1;
   if( do_start_files( &start_files ) ) return 1;
 
+  /* Must do this after all subsytems are initialised */
+  debugger_command_evaluate( settings_current.debugger_command );
+
+  if( ui_mouse_present ) ui_mouse_grabbed = ui_mouse_grab( 1 );
+
   fuse_emulation_paused = 0;
 
   return 0;
-
 }
 
 static
@@ -284,15 +316,9 @@ int creator_init( void )
 {
   size_t i;
   unsigned int version[4] = { 0, 0, 0, 0 };
-  char *custom;
+  char *custom, osname[ 256 ];
+  static const size_t CUSTOM_SIZE = 256;
   
-#ifndef WIN32
-  struct utsname buf;
-#else				/* #ifndef WIN32 */
-  OSVERSIONINFO buf;
-  char *windows_name;
-#endif				/* #ifndef WIN32 */
-
   libspectrum_error error; int sys_error;
 
   const char *gcrypt_version;
@@ -302,23 +328,10 @@ int creator_init( void )
 
   for( i=0; i<4; i++ ) if( version[i] > 0xff ) version[i] = 0xff;
 
-#ifndef WIN32
-  sys_error = uname( &buf );
-  if( sys_error == -1 ) {
-    ui_error( UI_ERROR_ERROR, "error getting system information: %s",
-	      strerror( errno ) );
-    return 1;
-  }
-#else				/* #ifndef WIN32 */
-  buf.dwOSVersionInfoSize = sizeof( buf );
-  sys_error = GetVersionEx( &buf );
-  if( sys_error == 0 ) {
-    ui_error( UI_ERROR_ERROR, "error getting system information." );
-    return 1;
-  }
-#endif				/* #ifndef WIN32 */
+  sys_error = compat_osname( osname, sizeof( osname ) );
+  if( sys_error ) return 1;
 
-  error = libspectrum_creator_alloc( &fuse_creator ); if( error ) return error;
+  fuse_creator = libspectrum_creator_alloc();
 
   error = libspectrum_creator_set_program( fuse_creator, "Fuse" );
   if( error ) { libspectrum_creator_free( fuse_creator ); return error; }
@@ -331,7 +344,7 @@ int creator_init( void )
 					 version[2] * 0x100 + version[3] );
   if( error ) { libspectrum_creator_free( fuse_creator ); return error; }
 
-  custom = malloc( 256 );
+  custom = malloc( CUSTOM_SIZE );
   if( !custom ) {
     ui_error( UI_ERROR_ERROR, "out of memory at %s:%d", __FILE__, __LINE__ );
     libspectrum_creator_free( fuse_creator );
@@ -341,31 +354,13 @@ int creator_init( void )
   gcrypt_version = libspectrum_gcrypt_version();
   if( !gcrypt_version ) gcrypt_version = "not available";
 
-#ifndef WIN32	    
+  snprintf( custom, CUSTOM_SIZE,
+	    "gcrypt: %s\nlibspectrum: %s\nuname: %s", gcrypt_version,
+	    libspectrum_version(), osname );
 
-  snprintf( custom, 256, "gcrypt: %s\nlibspectrum: %s\nuname: %s %s %s",
-	    gcrypt_version, libspectrum_version(),
-	    buf.sysname, buf.machine, buf.release );
-
-#else				/* #ifndef WIN32 */
-
-  switch( buf.dwPlatformId ) {
-  case VER_PLATFORM_WIN32_NT:	   windows_name = "NT";      break;
-  case VER_PLATFORM_WIN32_WINDOWS: windows_name = "95/98";   break;
-  case VER_PLATFORM_WIN32s:	   windows_name = "3.1";     break;
-  default:			   windows_name = "unknown"; break;
-  }
-
-  snprintf( custom, 256,
-	    "gcrypt: %s\nlibspectrum: %s\nuname: Windows %s %d.%d build %d %s",
-	    gcrypt_version, libspectrum_version(),
-	    windows_name, buf.dwMajorVersion, buf.dwMinorVersion,
-	    buf.dwBuildNumber, buf.szCSDVersion );
-
-#endif				/* #ifndef WIN32 */
-
-  error = libspectrum_creator_set_custom( fuse_creator,
-					  custom, strlen( custom ) );
+  error = libspectrum_creator_set_custom(
+    fuse_creator, (libspectrum_byte*)custom, strlen( custom )
+  );
   if( error ) {
     free( custom ); libspectrum_creator_free( fuse_creator );
     return error;
@@ -379,8 +374,11 @@ static void fuse_show_copyright(void)
   printf( "\n" );
   fuse_show_version();
   printf(
-   "Copyright (c) 1999-2004 Philip Kendall <pak21-fuse@srcf.ucam.org>\n"
-   "and others; see the file 'AUTHORS' for more details.\n"
+   "Copyright (c) 1999-2009 Philip Kendall and others; see the file\n"
+   "'AUTHORS' for more details.\n"
+   "\n"
+   "For help, please mail <fuse-emulator-devel@lists.sf.net> or use\n"
+   "the forums at <http://sourceforge.net/forum/?group_id=91293>.\n"
    "\n"
    "This program is distributed in the hope that it will be useful,\n"
    "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
@@ -441,7 +439,7 @@ int fuse_emulation_pause(void)
       
   /* If we had sound enabled (and hence doing the speed regulation),
      turn it off */
-  if( sound_enabled ) sound_end();
+  sound_pause();
 
   return 0;
 }
@@ -456,45 +454,7 @@ int fuse_emulation_unpause(void)
   if( --fuse_emulation_paused ) return 0;
 
   /* If we now want sound, enable it */
-  if( sound_enabled_ever && settings_current.sound &&
-      settings_current.emulation_speed == 100 ) {
-
-    /* If sound has just started providing the timing, remove the old
-       SIGALRM timer */
-#ifndef ALWAYS_USE_TIMER
-    if( !fuse_sound_in_use ) timer_end();
-#endif				/* #ifndef ALWAYS_USE_TIMER */
-
-    sound_init( settings_current.sound_device );
-
-    /* If the sound code couldn't re-initialise, fall back to the
-       signal based routines */
-    if( !sound_enabled ) {
-      /* Increment pause_count, report, decrement pause_count
-       * (i.e. avoid the effects of fuse_emulation_{,un}pause).
-       * Otherwise, we may be recursively reporting this error. */
-      fuse_emulation_paused++;
-      fuse_emulation_paused--;
-      settings_current.sound = fuse_sound_in_use = 0;
-      /* FIXME: How to deal with error return here? */
-#ifndef ALWAYS_USE_TIMER
-      timer_init();
-#endif				/* #ifndef ALWAYS_USE_TIMER */
-
-    }
-#ifdef ALWAYS_USE_TIMER
-    timer_init();
-#endif				/* #ifdef ALWAYS_USE_TIMER */
-    fuse_sound_in_use = sound_enabled;
-  }
-#ifndef ALWAYS_USE_TIMER
-  /* If we don't want sound any more, put previously did, start the SIGALRM
-     timer */
-  else if( fuse_sound_in_use ) {
-    timer_init();
-    fuse_sound_in_use = 0;
-  }
-#endif				/* #ifndef ALWAYS_USE_TIMER */
+  sound_unpause();
 
   /* Restart speed estimation with no information */
   error = timer_estimate_reset(); if( error ) return error;
@@ -506,7 +466,8 @@ static int
 setup_start_files( start_files_t *start_files )
 {
   start_files->disk_plus3 = settings_current.plus3disk_file;
-  start_files->disk_trdos = settings_current.trdosdisk_file;
+  start_files->disk_plusd = settings_current.plusddisk_file;
+  start_files->disk_beta = settings_current.betadisk_file;
   start_files->dock = settings_current.dck_file;
   start_files->if2 = settings_current.if2_file;
   start_files->playback = settings_current.playback_file;
@@ -522,6 +483,18 @@ setup_start_files( start_files_t *start_files )
 
   start_files->zxcf = settings_current.zxcf_pri_file;
 
+  start_files->divide_master = settings_current.divide_master_file;
+  start_files->divide_slave  = settings_current.divide_slave_file;
+
+  start_files->mdr[0] = settings_current.mdr_file;
+  start_files->mdr[1] = settings_current.mdr_file2;
+  start_files->mdr[2] = settings_current.mdr_file3;
+  start_files->mdr[3] = settings_current.mdr_file4;
+  start_files->mdr[4] = settings_current.mdr_file5;
+  start_files->mdr[5] = settings_current.mdr_file6;
+  start_files->mdr[6] = settings_current.mdr_file7;
+  start_files->mdr[7] = settings_current.mdr_file8;
+
   return 0;
 }
 
@@ -530,7 +503,7 @@ static int
 parse_nonoption_args( int argc, char **argv, int first_arg,
 		      start_files_t *start_files )
 {
-  size_t i;
+  size_t i, j;
   const char *filename;
   utils_file file;
   libspectrum_id_t type;
@@ -563,6 +536,8 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
 	start_files->zxatasp_master = filename;
       } else if( settings_current.simpleide_active ) {
 	start_files->simpleide_master = filename;
+      } else if( settings_current.divide_enabled ) {
+	start_files->divide_master = filename;
       } else {
 	/* No IDE interface active, so activate the ZXCF */
 	settings_current.zxcf_active = 1;
@@ -573,8 +548,28 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
     case LIBSPECTRUM_CLASS_DISK_PLUS3:
       start_files->disk_plus3 = filename; break;
 
+    case LIBSPECTRUM_CLASS_DISK_PLUSD:
+      start_files->disk_plusd = filename; break;
+
     case LIBSPECTRUM_CLASS_DISK_TRDOS:
-      start_files->disk_trdos = filename; break;
+      start_files->disk_beta = filename; break;
+
+    case LIBSPECTRUM_CLASS_DISK_GENERIC:
+      if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_PLUS2A )
+        start_files->disk_plus3 = filename;
+      else if( machine_current->machine == LIBSPECTRUM_MACHINE_PENT ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_PENT512 ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_PENT1024 ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_SCORP )
+        start_files->disk_beta = filename; 
+      else {
+        if( periph_beta128_active )
+          start_files->disk_beta = filename; 
+        else if( periph_plusd_active )
+          start_files->disk_plusd = filename;
+      }
+      break;
 
     case LIBSPECTRUM_CLASS_RECORDING:
       start_files->playback = filename; break;
@@ -582,6 +577,15 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
     case LIBSPECTRUM_CLASS_SNAPSHOT:
       start_files->snapshot = filename; break;
 
+    case LIBSPECTRUM_CLASS_MICRODRIVE:
+      for( j = 0; j < 8; j++ ) {
+        if( !start_files->mdr[j] ) {
+	  start_files->mdr[j] = filename;
+	  break;
+	}
+      }
+      break;
+
     case LIBSPECTRUM_CLASS_TAPE:
       start_files->tape = filename; break;
 
@@ -604,7 +608,7 @@ parse_nonoption_args( int argc, char **argv, int first_arg,
 static int
 do_start_files( start_files_t *start_files )
 {
-  int autoload, error;
+  int autoload, error, i;
 
   /* Can't do both input recording and playback */
   if( start_files->playback && start_files->recording ) {
@@ -616,7 +620,7 @@ do_start_files( start_files_t *start_files )
   }
 
   /* Can't use both +3 and TR-DOS disks simultaneously */
-  if( start_files->disk_plus3 && start_files->disk_trdos ) {
+  if( start_files->disk_plus3 && start_files->disk_beta ) {
     ui_error(
       UI_ERROR_WARNING,
       "can't use +3 and TR-DOS disks simultaneously; +3 disk ignored"
@@ -625,7 +629,7 @@ do_start_files( start_files_t *start_files )
   }
 
   /* Can't use disks and the dock simultaneously */
-  if( ( start_files->disk_plus3 || start_files->disk_trdos ) &&
+  if( ( start_files->disk_plus3 || start_files->disk_beta ) &&
       start_files->dock                                         ) {
     ui_error(
       UI_ERROR_WARNING,
@@ -635,7 +639,7 @@ do_start_files( start_files_t *start_files )
   }
 
   /* Can't use disks and the Interface II simultaneously */
-  if( ( start_files->disk_plus3 || start_files->disk_trdos ) &&
+  if( ( start_files->disk_plus3 || start_files->disk_beta ) &&
       start_files->if2                                          ) {
     ui_error(
       UI_ERROR_WARNING,
@@ -645,7 +649,7 @@ do_start_files( start_files_t *start_files )
   }
 
   /* If a snapshot has been specified, don't autoload tape, disks etc */
-  autoload = start_files->snapshot ? 0 : settings_current.auto_load;
+  autoload = start_files->snapshot ? 0 : tape_can_autoload();
 
   /* Load in each of the files. Input recording must be done after
      snapshot loading such that the right snapshot is embedded into
@@ -658,8 +662,13 @@ do_start_files( start_files_t *start_files )
     if( error ) return error;
   }
 
-  if( start_files->disk_trdos ) {
-    error = utils_open_file( start_files->disk_trdos, autoload, NULL );
+  if( start_files->disk_plusd ) {
+    error = utils_open_file( start_files->disk_plusd, autoload, NULL );
+    if( error ) return error;
+  }
+
+  if( start_files->disk_beta ) {
+    error = utils_open_file( start_files->disk_beta, autoload, NULL );
     if( error ) return error;
   }
 
@@ -683,19 +692,28 @@ do_start_files( start_files_t *start_files )
     if( error ) return error;
   }
 
+  /* Microdrive cartridges */
+
+  for( i = 0; i < 8; i++ ) {
+    if( start_files->mdr[i] ) {
+      error = utils_open_file( start_files->mdr[i], autoload, NULL );
+      if( error ) return error;
+    }
+  }
+
   /* IDE hard disk images */
 
   if( start_files->simpleide_master ) {
     error = simpleide_insert( start_files->simpleide_master,
 			      LIBSPECTRUM_IDE_MASTER );
-    simpleide_reset();
+    simpleide_reset( 0 );
     if( error ) return error;
   }
 
   if( start_files->simpleide_slave ) {
     error = simpleide_insert( start_files->simpleide_slave,
 			      LIBSPECTRUM_IDE_SLAVE );
-    simpleide_reset();
+    simpleide_reset( 0 );
     if( error ) return error;
   }
 
@@ -715,6 +733,18 @@ do_start_files( start_files_t *start_files )
     error = zxcf_insert( start_files->zxcf ); if( error ) return error;
   }
 
+  if( start_files->divide_master ) {
+    error = divide_insert( start_files->divide_master,
+			    LIBSPECTRUM_IDE_MASTER );
+    if( error ) return error;
+  }
+
+  if( start_files->divide_slave ) {
+    error = divide_insert( start_files->divide_slave,
+			    LIBSPECTRUM_IDE_SLAVE );
+    if( error ) return error;
+  }
+
   /* Input recordings */
 
   if( start_files->playback ) {
@@ -738,32 +768,33 @@ static int fuse_end(void)
      set from memory for the text output */
   printer_end();
 
+  /* also required before memory is deallocated on Fuse for OS X where
+     settings need to look up machine names etc. */
+  settings_end();
+
   psg_end();
   rzx_end();
   debugger_end();
   simpleide_end();
   zxatasp_end();
   zxcf_end();
+  if1_end();
+  divide_end();
+  plusd_end();
 
   machine_end();
 
-#ifdef ALWAYS_USE_TIMER
   timer_end();
-#else				/* #ifdef ALWAYS_USE_TIMER */
-  if(!sound_enabled) timer_end();
-#endif				/* #ifdef ALWAYS_USE_TIMER */
 
   sound_end();
   event_end();
-  fuse_joystick_end ();
+  fuse_joystick_end();
   ui_end();
 
 #ifdef USE_WIDGET
   widget_end();
 #endif                          /* #ifdef USE_WIDGET */
 
-  settings_end();
-
   libspectrum_creator_free( fuse_creator );
 
   return 0;
