@@ -1,10 +1,10 @@
 #!/usr/bin/perl -w
 
 # keysyms.pl: generate keysyms.c from keysyms.dat
-# Copyright (c) 2000-2004 Philip Kendall, Matan Ziv-Av, Russell Marks,
-#			  Fredrick Meunier
+# Copyright (c) 2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks,
+#			  Fredrick Meunier, Catalin Mihaila, Stuart Brady
 
-# $Id: keysyms.pl,v 1.17 2004/04/08 18:43:20 pak21 Exp $
+# $Id: keysyms.pl 3735 2008-07-28 21:27:56Z specu $
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -16,14 +16,13 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 49 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 # Author contact information:
 
-# E-mail: pak21-fuse@srcf.ucam.org
-# Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+# E-mail: philip-fuse@shadowmagic.org.uk
 
 use strict;
 
@@ -35,7 +34,7 @@ my $ui = shift;
 $ui = 'gtk' unless defined $ui;
 
 die "$0: unrecognised user interface: $ui\n"
-  unless 0 < grep { $ui eq $_ } ( 'ggi', 'gtk', 'x', 'svga', 'fb', 'sdl' );
+  unless 0 < grep { $ui eq $_ } ( 'gtk', 'x', 'svga', 'fb', 'sdl', 'win32' );
 
 sub fb_keysym ($) {
 
@@ -47,53 +46,6 @@ sub fb_keysym ($) {
     return $keysym;
 }
 
-# The GGI keysyms which start with `GIIK', not `GIIUC'
-my %ggi_giik_keysyms = map { $_ => 1 } qw(
-
-    AltL
-    AltR
-    CapsLock
-    CtrlL
-    CtrlR
-    Down
-    End
-    Home
-    HyperL
-    HyperR
-    Left
-    Menu
-    MetaL
-    MetaR
-    PageUp
-    PageDown
-    Right
-    ShiftL
-    ShiftR
-    SuperL
-    SuperR
-    Up
-
-);
-
-sub ggi_keysym ($) {
-
-    my $keysym = shift;
-
-    # GGI 'long name' keysyms start with a capitial letter
-    $keysym = ucfirst $keysym if length $keysym > 1;
-
-    # and have no underscores
-    $keysym =~ s/_//g;
-
-    if( $ggi_giik_keysyms{ $keysym } ) {
-	$keysym = "GIIK_$keysym";
-    } else {
-	$keysym = "GIIUC_$keysym";
-    }
-
-    return $keysym;
-}
-
 sub sdl_keysym ($) {
 
     my $keysym = shift;
@@ -110,6 +62,18 @@ sub sdl_keysym ($) {
     return $keysym;
 }
 
+sub sdl_unicode_keysym ($) {
+
+    my $keysym = shift;
+
+    if ( $keysym eq "'" ) {
+        $keysym = "\\'";
+    }
+    $keysym = "'$keysym'";
+
+    return $keysym;
+}
+
 sub svga_keysym ($) {
     
     my $keysym = shift;
@@ -129,6 +93,28 @@ sub svga_keysym ($) {
     return $keysym;
 }
 
+sub win32_keysym ($) {
+
+    my $keysym = shift;
+
+    # http://msdn2.microsoft.com/en-us/library/ms645540.aspx
+
+    $keysym =~ tr/a-z/A-Z/;
+
+    if ( $keysym =~ /^[A-Z0-9]$/ ) {
+	$keysym = "'$keysym'";
+    } else {
+	$keysym =~ s/(.*)_L$/$1/;
+	$keysym =~ s/META$/LWIN/;
+	$keysym =~ s/ALT$/MENU/;	# Not a typo: the 'menu' key is VK_APPS
+
+	# All the magic #defines start with `VK_'
+	$keysym = "VK_$keysym";
+    }
+
+    return $keysym;
+}
+
 # Parameters for each UI
 my %ui_data = (
 
@@ -141,19 +127,6 @@ my %ui_data = (
 	      function => \&fb_keysym
 	    },
 
-    ggi  => { headers => [ 'ggi/ggi.h' ],
-	      max_length => 18,
-	      skips => { },
-	      translations => {
-	          Control_L   => 'CtrlL',
-		  Control_R   => 'CtrlR',
-		  Mode_switch => 'Menu',
-		  apostrophe  => 'DoubleQuote',
-		  numbersign  => 'Hash',
-	      },
-	      function => \&ggi_keysym,
-	    },
-
     gtk  => { headers => [ 'gdk/gdkkeysyms.h' ],
 	      max_length => 16,
 	      skips => { },
@@ -163,10 +136,16 @@ my %ui_data = (
 
     sdl  => { headers => [ 'SDL.h' ],
 	      max_length => 15,
-	      skips => { map { $_ => 1 } qw( Hyper_L Hyper_R ) },
+	      skips => { map { $_ => 1 } ( 'Hyper_L','Hyper_R','Caps_Lock',
+                         'A' .. 'Z' ) },
+	      unicode_skips => { map { $_ => 1 } qw( Hyper_L Hyper_R Caps_Lock
+                         Escape F1 F2 F3 F4 F5 F6 F7 F8 F9 F10 F11 F12
+                         BackSpace Tab Caps_Lock Return Shift_L Shift_R
+                         Control_L Control_R Alt_L Alt_R Meta_L Meta_R
+                         Super_L Super_R Mode_switch Up Down Left Right
+                         Insert Delete Home End Page_Up Page_Down ) },
 	      translations => {
 		  apostrophe  => 'QUOTE',
-		  Caps_Lock   => 'CAPSLOCK',
 		  Control_L   => 'LCTRL',	 
 		  Control_R   => 'RCTRL',	 
 		  equal       => 'EQUALS',
@@ -175,7 +154,19 @@ my %ui_data = (
 		  Page_Up     => 'PAGEUP',
 		  Page_Down   => 'PAGEDOWN',
 	      },
+	      unicode_translations => {
+                  space       => ' ',
+                  numbersign  => '#',
+                  apostrophe  => "'",
+                  comma       => ',',
+                  minus       => '-',
+                  period      => '.',
+                  slash       => '/',
+                  semicolon   => ';',
+                  equal       => '=',
+	      },
 	      function => \&sdl_keysym,
+	      unicode_function => \&sdl_unicode_keysym,
 	    },
 
     svga => { headers => [ 'vgakeyboard.h' ],
@@ -186,6 +177,7 @@ my %ui_data = (
 		  Caps_Lock  => 'CAPSLOCK',
 		  numbersign => 'BACKSLASH',
 		  Return     => 'ENTER',
+		  Delete     => 'REMOVE',
 		  Left       => 'CURSORBLOCKLEFT',
 		  Down       => 'CURSORBLOCKDOWN',
 		  Up         => 'CURSORBLOCKUP',
@@ -200,6 +192,31 @@ my %ui_data = (
 	      translations => { },
 	      function => sub ($) { "XK_$_[0]" },
 	    },
+
+    win32 => { headers => [ 'windows.h' ],
+	      max_length => 16,
+	      skips => { map { $_ => 1 } ( 'Shift_R','Control_R',
+					   'Alt_R','Meta_R',
+					   'Hyper_L','Hyper_R',
+					   'Super_L','Super_R',
+					   'A' .. 'Z' ) },
+	      translations => { 
+		  numbersign  => 'OEM_5',
+		  apostrophe  => 'OEM_7',
+		  comma       => 'OEM_COMMA',
+		  minus       => 'OEM_MINUS',
+		  period      => 'OEM_PERIOD',
+		  slash       => 'OEM_2',
+		  BackSpace   => 'BACK',
+		  semicolon   => 'OEM_1',
+		  equal       => 'OEM_PLUS',
+		  Page_Up     => 'PRIOR',
+		  Page_Down   => 'NEXT',
+		  Caps_Lock   => 'CAPITAL',
+	          Mode_switch => 'APPS',
+	      },
+	      function => \&win32_keysym
+	    },
 );
 
 # Translation table for any UI which uses keyboard mode K_MEDIUMRAW
@@ -248,8 +265,8 @@ my $define = uc $ui;
 
 print Fuse::GPL(
     'keysyms.c: UI keysym to Fuse input layer keysym mappings',
-    "2000-2004 Philip Kendall, Matan Ziv-Av, Russell Marks,\n" .
-    "                           Fredrick Meunier, Catalin Mihaila"  ),
+    "2000-2007 Philip Kendall, Matan Ziv-Av, Russell Marks\n" .
+    "                           Fredrick Meunier, Catalin Mihaila, Stuart Brady" ),
     << "CODE";
 
 /* This file is autogenerated from keysyms.dat by keysyms.pl.
@@ -257,7 +274,25 @@ print Fuse::GPL(
 
 #include <config.h>
 
-#ifdef UI_$define
+CODE
+
+# Comment to unbreak Emacs' perl mode
+
+if ( $ui eq 'win32' ) {
+    # To allow VK_OEM_{COMMA,MINUS,PERIOD,PLUS} to be used, we must
+    # define WINVER to be >= 0x0500 before we include <windows.h>
+    # But instead of requiring Windows 2000 or later, it's better to
+    # define the missing values
+    print "#if ! (_WIN32_WINNT >= 0x0500)\n"
+        . "#define VK_OEM_PLUS     0xBB\n"
+        . "#define VK_OEM_COMMA    0xBC\n"
+        . "#define VK_OEM_MINUS    0xBD\n"
+        . "#define VK_OEM_PERIOD   0xBE\n"
+        . "#endif\n"
+        . "\n";
+}
+
+print << "CODE";
 
 #include "input.h"
 #include "keyboard.h"
@@ -293,12 +328,10 @@ foreach( @keys ) {
     if( $ui eq 'fb' ) {
 
 	for( my $i = 0; $i <= $#cooked_keysyms; $i++ ) {
-	    if( defined $cooked_keysyms[$i] and
-		$cooked_keysyms[$i] eq $ui_keysym ) {
-#		printf "  { %3i, KEYBOARD_%-9s KEYBOARD_%-6s },\n", $i,
-#		    "$key1,", $key2;
-		last;
-	    }
+	    next unless defined $cooked_keysyms[$i] and
+			$cooked_keysyms[$i] eq $ui_keysym;
+	    printf "  { %3i, INPUT_KEY_%-11s },\n", $i, $keysym;
+	    last;
 	}
 
     } else {
@@ -306,11 +339,6 @@ foreach( @keys ) {
 	printf "  { %-$ui_data{$ui}{max_length}s INPUT_KEY_%-11s },\n",
             "$ui_keysym,", $keysym;
 
-	if( $ui eq 'ggi' and $ui_keysym =~ /GIIUC_[a-z]/ ) {
-#	    printf "  { %-$ui_data{$ui}{max_length}s KEYBOARD_%-9s KEYBOARD_%-6s },\n",
-#	    uc $keysym . ",", "$key1,", $key2;
-	}
-
     }
 
     if( $ui eq 'svga' and $ui_keysym =~ /WIN$/ ) {
@@ -325,5 +353,35 @@ print << "CODE";
 
 };
 
-#endif
 CODE
+
+if( $ui eq 'sdl' ) {
+
+print "\nkeysyms_map_t unicode_keysyms_map[] = {\n\n";
+
+    foreach( @keys ) {
+
+        my( $keysym ) = @$_;
+
+        next if $ui_data{$ui}{unicode_skips}{$keysym};
+
+        my $ui_keysym = $keysym;
+
+        $ui_keysym = $ui_data{$ui}{unicode_translations}{$keysym} if
+            $ui_data{$ui}{unicode_translations}{$keysym};
+
+        $ui_keysym = $ui_data{$ui}{unicode_function}->( $ui_keysym );
+
+	printf "  { %-$ui_data{$ui}{max_length}s INPUT_KEY_%-11s },\n",
+            "$ui_keysym,", $keysym;
+
+    }
+
+print << "CODE";
+
+  { 0, 0 }			/* End marker: DO NOT MOVE! */
+
+};
+
+CODE
+}
