@@ -1,7 +1,7 @@
 /* memory.h: memory access routines
    Copyright (c) 2003-2004 Philip Kendall
 
-   $Id: memory.h,v 1.18 2004/06/06 22:31:14 pak21 Exp $
+   $Id: memory.h 3603 2008-04-15 13:33:24Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -46,6 +45,8 @@ typedef enum memory_page_source {
 
   MEMORY_SOURCE_SYSTEM,
   MEMORY_SOURCE_CARTRIDGE,
+  MEMORY_SOURCE_PERIPHERAL,
+  MEMORY_SOURCE_CUSTOMROM,
   
 } memory_page_source;
 
@@ -96,9 +97,8 @@ const char *memory_bank_name( memory_page *page );
 /* Map in alternate bank if ROMCS is set */
 void memory_romcs_map( void );
 
-/* Copy RAM to and from snapshots */
-int memory_ram_from_snapshot( libspectrum_snap *snap, int capabilities );
-int memory_ram_to_snapshot( libspectrum_snap *snap );
+/* Have we loaded any custom ROMs? */
+int memory_custom_rom( void );
 
 libspectrum_byte readbyte( libspectrum_word address );
 
