@@ -1,7 +1,7 @@
 /* rzx.c: .rzx files
    Copyright (c) 2002-2003 Philip Kendall
 
-   $Id: rzx.c,v 1.61 2004/07/16 08:38:41 pak21 Exp $
+   $Id: rzx.c 3836 2008-11-18 04:24:34Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -34,6 +33,7 @@
 #include <windows.h>
 #endif				/* #ifdef WIN32 */
 
+#include "debugger/debugger.h"
 #include "event.h"
 #include "fuse.h"
 #include "machine.h"
@@ -41,12 +41,16 @@
 #include "rzx.h"
 #include "settings.h"
 #include "snapshot.h"
-#include "timer.h"
+#include "timer/timer.h"
 #include "ui/ui.h"
+#include "ula.h"
 #include "utils.h"
 #include "z80/z80.h"
 #include "z80/z80_macros.h"
 
+#define RZX_SENTINEL_TIME ( ULA_CONTENTION_SIZE - 1000 )
+#define RZX_SENTINEL_TIME_REDUCE 8000
+
 /* The offset used to get the count of instructions from the R register;
    (instruction count) = R + rzx_instructions_offset */
 int rzx_instructions_offset;
@@ -54,6 +58,9 @@ int rzx_instructions_offset;
 /* The number of bytes read via IN during the current frame */
 size_t rzx_in_count;
 
+/* The number of frames we've recorded in this RZX file */
+static size_t autosave_frame_count;
+
 /* And the values of those bytes */
 libspectrum_byte *rzx_in_bytes;
 
@@ -69,9 +76,6 @@ int rzx_competition_mode;
 /* The filename we'll save this recording into */
 static char *rzx_filename;
 
-/* The snapshot taken when we started the recording */
-static libspectrum_snap *rzx_snap;
-
 /* Are we currently playing back a .rzx file? */
 int rzx_playback;
 
@@ -90,21 +94,27 @@ libspectrum_rzx_dsa_key rzx_key = {
   "948744AA7A1D1BE9EE65150B0A95A678B4181F0E"	       /* x */
 };
 
-/* The time we started recording this RZX file */
-static timer_type start_time;
-
-/* How many 1/50ths of a second do we expect to have elapsed since we
-   started recording? */
-static int expected_time;
-
 /* By how much is the speed allowed to deviate from 100% whilst recording
    a competition mode RZX file */
-static const float SPEED_TOLERANCE = 0.05;
+static const float SPEED_TOLERANCE = 5;
 
-static int start_playback( libspectrum_rzx *rzx, libspectrum_snap *snap );
+/* How often will we create an autosave file */
+static const size_t AUTOSAVE_INTERVAL = 5 * 50;
+
+/* Debugger events */
+static const char *event_type_string = "rzx";
+static const char *end_event_detail_string = "end";
+
+int end_event;
+
+static int start_playback( libspectrum_rzx *rzx );
 static int recording_frame( void );
 static int playback_frame( void );
 static int counter_reset( void );
+static void rzx_sentinel( libspectrum_dword tstates, int type,
+			  void *user_data );
+
+static int sentinel_event;
 
 int rzx_init( void )
 {
@@ -112,18 +122,23 @@ int rzx_init( void )
 
   rzx_in_bytes = NULL;
   rzx_in_allocated = 0;
-  rzx_snap = NULL;
+
+  sentinel_event = event_register( rzx_sentinel, "RZX sentinel" );
+  if( sentinel_event == -1 ) return 1;
+
+  end_event = debugger_event_register( event_type_string, end_event_detail_string );
+  if( end_event == -1 ) return 1;
 
   return 0;
 }
 
 int rzx_start_recording( const char *filename, int embed_snapshot )
 {
-  int error; libspectrum_error libspec_error;
+  int error;
 
   if( rzx_playback ) return 1;
 
-  error = libspectrum_rzx_alloc( &rzx ); if( error ) return error;
+  rzx = libspectrum_rzx_alloc();
 
   /* Store the filename */
   rzx_filename = strdup( filename );
@@ -134,25 +149,29 @@ int rzx_start_recording( const char *filename, int embed_snapshot )
 
   /* If we're embedding a snapshot, create it now */
   if( embed_snapshot ) {
-    
-    libspec_error = libspectrum_snap_alloc( &rzx_snap );
-    if( libspec_error != LIBSPECTRUM_ERROR_NONE ) return 1;
 
-    error = snapshot_copy_to( rzx_snap );
+    libspectrum_snap *snap = libspectrum_snap_alloc();
+
+    error = snapshot_copy_to( snap );
     if( error ) {
-      libspectrum_snap_free( rzx_snap ); rzx_snap = NULL;
+      libspectrum_snap_free( snap );
       return 1;
     }
-  } else {
-    rzx_snap = NULL;
+
+    error = libspectrum_rzx_add_snap( rzx, snap, 0 );
+    if( error ) {
+      libspectrum_snap_free( snap );
+      return error;
+    }
+
   }
 
-  /* Start the count of instruction fetches here */
-  counter_reset(); rzx_in_count = 0;
+  libspectrum_rzx_start_input( rzx, tstates );
 
-  libspectrum_rzx_set_tstates( rzx, tstates );
+  counter_reset();
+  rzx_in_count = 0;
+  autosave_frame_count = 0;
 
-  /* Note that we're recording */
   rzx_recording = 1;
 
   ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
@@ -163,13 +182,12 @@ int rzx_start_recording( const char *filename, int embed_snapshot )
       ui_error( UI_ERROR_WARNING,
 		"gcrypt not available: recording will NOT be signed" );
 
-    expected_time = 0;
     settings_current.emulation_speed = 100;
-    timer_count = 0.0;
     rzx_competition_mode = 1;
     
   } else {
 
+    ui_menu_activate( UI_MENU_ITEM_RECORDING_ROLLBACK, 1 );
     rzx_competition_mode = 0;
 
   }
@@ -188,24 +206,22 @@ int rzx_stop_recording( void )
   rzx_recording = 0;
 
   ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING_ROLLBACK, 0 );
 
   libspectrum_creator_set_competition_code(
     fuse_creator, settings_current.competition_code
   );
 
   length = 0;
-  libspec_error = libspectrum_rzx_write2(
-    &buffer, &length, rzx, rzx_snap, LIBSPECTRUM_ID_UNKNOWN, fuse_creator,
+  libspec_error = libspectrum_rzx_write(
+    &buffer, &length, rzx, LIBSPECTRUM_ID_UNKNOWN, fuse_creator,
     settings_current.rzx_compression, rzx_competition_mode ? &rzx_key : NULL
   );
   if( libspec_error != LIBSPECTRUM_ERROR_NONE ) {
     libspectrum_rzx_free( rzx );
-    if( rzx_snap ) libspectrum_snap_free( rzx_snap );
     return libspec_error;
   }
 
-  if( rzx_snap ) libspectrum_snap_free( rzx_snap );
-
   error = utils_write_file( rzx_filename, buffer, length );
   free( rzx_filename );
   if( error ) { free( buffer ); libspectrum_rzx_free( rzx ); return error; }
@@ -222,17 +238,15 @@ int rzx_start_playback( const char *filename )
 {
   utils_file file;
   libspectrum_error libspec_error; int error;
-  libspectrum_snap *snap;
 
   if( rzx_recording ) return 1;
 
-  error = libspectrum_rzx_alloc( &rzx ); if( error ) return error;
+  rzx = libspectrum_rzx_alloc();
 
   error = utils_read_file( filename, &file );
   if( error ) return error;
 
-  libspec_error = libspectrum_rzx_read( rzx, &snap, file.buffer, file.length,
-					NULL );
+  libspec_error = libspectrum_rzx_read( rzx, file.buffer, file.length );
   if( libspec_error != LIBSPECTRUM_ERROR_NONE ) {
     utils_close_file( &file );
     return libspec_error;
@@ -240,73 +254,60 @@ int rzx_start_playback( const char *filename )
 
   if( utils_close_file( &file ) ) {
     libspectrum_rzx_free( rzx );
-    if( snap ) libspectrum_snap_free( snap );
     return 1;
   }
 
-  if( !snap ) {
-    error = menu_open_snap();
-    if( error ) { libspectrum_rzx_free( rzx ); return 1; }
-  }
-
-  error = start_playback( rzx, snap );
+  error = start_playback( rzx );
   if( error ) {
-    if( snap ) libspectrum_snap_free( snap );
     libspectrum_rzx_free( rzx );
     return error;
   }
 
-  if( snap ) {
-    error = libspectrum_snap_free( snap );
-    if( error ) { libspectrum_rzx_free( rzx ); return error; }
-  }
-
   return 0;
 }
 
 int
 rzx_start_playback_from_buffer( const unsigned char *buffer, size_t length )
 {
-  libspectrum_snap *snap;
   int error;
 
   if( rzx_recording ) return 0;
 
-  error = libspectrum_rzx_alloc( &rzx ); if( error ) return error;
+  rzx = libspectrum_rzx_alloc();
 
-  error = libspectrum_rzx_read( rzx, &snap, buffer, length, NULL );
+  error = libspectrum_rzx_read( rzx, buffer, length );
   if( error ) return error;
 
-  error = start_playback( rzx, snap );
+  error = start_playback( rzx );
   if( error ) {
-    if( snap ) libspectrum_snap_free( snap );
     libspectrum_rzx_free( rzx );
     return error;
   }
 
-  if( snap ) {
-    error = libspectrum_snap_free( snap );
-    if( error ) { libspectrum_rzx_free( rzx ); return error; }
-  }
-
   return 0;
 }
 
 static int
-start_playback( libspectrum_rzx *rzx, libspectrum_snap *snap )
+start_playback( libspectrum_rzx *rzx )
 {
   int error;
+  libspectrum_snap *snap;
 
-  if( snap ) {
-    error = snapshot_copy_from( snap ); if( error ) return error;
-  }
-  
-  /* End of frame will now be generated by the RZX code */
-  error = event_remove_type( EVENT_TYPE_FRAME );
+  error = libspectrum_rzx_start_playback( rzx, 0, &snap );
   if( error ) return error;
 
-  /* We're now playing this RZX file */
-  if( libspectrum_rzx_start_playback( rzx ) ) return 1;
+  if( snap ) {
+    error = snapshot_copy_from( snap );
+    if( error ) return error;
+  }
+
+  /* End of frame will now be generated by the RZX code */
+  error = event_remove_type( spectrum_frame_event );
+  if( error ) return error;
+
+  /* Add a sentinel event to prevent tstates overrun (bug #1057471) */
+  error = event_add( RZX_SENTINEL_TIME, sentinel_event );
+  if( error ) return error;
 
   tstates = libspectrum_rzx_tstates( rzx );
   rzx_instruction_count = libspectrum_rzx_instructions( rzx );
@@ -314,6 +315,7 @@ start_playback( libspectrum_rzx *rzx, libspectrum_snap *snap )
   counter_reset();
 
   ui_menu_activate( UI_MENU_ITEM_RECORDING, 1 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING_ROLLBACK, 0 );
 
   return 0;
 }
@@ -325,20 +327,40 @@ int rzx_stop_playback( int add_interrupt )
   rzx_playback = 0;
 
   ui_menu_activate( UI_MENU_ITEM_RECORDING, 0 );
+  ui_menu_activate( UI_MENU_ITEM_RECORDING_ROLLBACK, 0 );
+
+  /* Remove our RZX sentinel event */
+  error = event_remove_type( sentinel_event );
+  if( error ) return error;
 
   /* We've now finished with the RZX file, so add an end of frame
      event if we've been requested to do so; we don't if we just run
      out of frames, as this occurs just before a normal end of frame
      and everything works normally as rzx_playback is now zero again */
   if( add_interrupt ) {
+
     error = event_add( machine_current->timings.tstates_per_frame,
-		       EVENT_TYPE_FRAME );
+		       spectrum_frame_event );
     if( error ) return error;
+
+    /* We're no longer doing RZX playback, so tstates now be <= the
+       normal frame count */
+    if( tstates > machine_current->timings.tstates_per_frame )
+      tstates = machine_current->timings.tstates_per_frame;
+
+  } else {
+
+    /* Ensure that tstates will be zero after it is reduced in
+       spectrum_frame() */
+    tstates = machine_current->timings.tstates_per_frame;
+
   }
 
   libspec_error = libspectrum_rzx_free( rzx );
   if( libspec_error != LIBSPECTRUM_ERROR_NONE ) return libspec_error;
 
+  debugger_event( end_event );
+
   return 0;
 }  
 
@@ -349,81 +371,121 @@ int rzx_frame( void )
   return 0;
 }
 
+typedef struct prune_info_t {
+  libspectrum_rzx_iterator it;
+  size_t frames;
+} prune_info_t;
+
+static void
+autosave_prune( void )
+{
+  GArray *autosaves = g_array_new( FALSE, FALSE, sizeof( prune_info_t ) );
+  libspectrum_rzx_iterator it;
+  size_t i, frames = 0;
+
+  for( it = libspectrum_rzx_iterator_begin( rzx );
+       it;
+       it = libspectrum_rzx_iterator_next( it ) ) {
+
+    libspectrum_rzx_block_id id = libspectrum_rzx_iterator_get_type( it );
+
+    switch( id ) {
+
+    case LIBSPECTRUM_RZX_INPUT_BLOCK:
+      frames += libspectrum_rzx_iterator_get_frames( it ); break;
+      
+    case LIBSPECTRUM_RZX_SNAPSHOT_BLOCK:
+      if( libspectrum_rzx_iterator_snap_is_automatic( it ) ) {
+        prune_info_t info = { it, frames };
+	g_array_append_val( autosaves, info );
+      }
+      break;
+
+    default:
+      break;
+    }
+  }
+
+  /* Convert 'time from start' into 'time before now' */
+  for( i = 0; i < autosaves->len; i++ ) {
+    prune_info_t *info = &( g_array_index( autosaves, prune_info_t, i ) );
+    info->frames = frames - info->frames;
+  }
+
+  for( i = autosaves->len - 1; i > 0; i-- ) {
+    prune_info_t save1 = g_array_index( autosaves, prune_info_t, i ),
+      save2 = g_array_index( autosaves, prune_info_t, i - 1 );
+
+    if( ( save1.frames == 15 * 50 ||
+          save1.frames == 60 * 50 ||
+	  save1.frames == 300 * 50   ) &&
+	save2.frames < 2 * save1.frames
+      )
+      /* FIXME: could possibly merge adjacent IRBs here */
+      libspectrum_rzx_iterator_delete( rzx, save1.it );
+  }
+
+  g_array_set_size( autosaves, 0 );
+}
+
+static void
+autosave_frame( void )
+{
+  libspectrum_snap *snap;
+
+  if( ++autosave_frame_count % AUTOSAVE_INTERVAL ) return;
+
+  snap = libspectrum_snap_alloc();
+  snapshot_copy_to( snap );
+
+  libspectrum_rzx_stop_input( rzx );
+  libspectrum_rzx_add_snap( rzx, snap, 1 );
+  libspectrum_rzx_start_input( rzx, tstates );
+
+  autosave_prune();
+}
+
 static int recording_frame( void )
 {
   libspectrum_error error;
 
   error = libspectrum_rzx_store_frame( rzx, R + rzx_instructions_offset,
 				       rzx_in_count, rzx_in_bytes );
-  if( error ) return error;
+  if( error ) {
+    rzx_stop_recording();
+    return error;
+  }
 
   /* Reset the instruction counter */
   rzx_in_count = 0; counter_reset();
 
   /* If we're in competition mode, check we're running at close to 100%
      speed */
-  if( rzx_competition_mode ) {
+  if( rzx_competition_mode && 
+      fabs( current_speed - 100.0 ) > SPEED_TOLERANCE ) {
 
-    expected_time++;
+    rzx_stop_recording();
 
-    /* We wait one second before starting time measurements at all.
-       This is to allow any initial speed fluctuations caused by the
-       menu system to settle down (notably, with a widget UI and OSS
-       sound, the first few frames run fast as the sound buffers fill
-       up) */
-    if( expected_time == 50 ) {
-
-      error = timer_get_real_time( &start_time );
-      if( error ) {
-	ui_error( UI_ERROR_ERROR, "stopping competition mode recording" );
-	rzx_stop_recording();
-	return error;
-      }
-      
-    /* Measuring small time intervals will be inaccurate, so don't
-       start checking for speed violations until at least one second
-       of measured time has elapsed (ie two seconds into recording) */
-    } else if( expected_time > 100 ) {
-
-      timer_type current_time;
-      float elapsed_seconds, expected_seconds;
-
-      error = timer_get_real_time( &current_time );
-      if( error ) {
-	ui_error( UI_ERROR_ERROR, "stopping competition mode recording" );
-	rzx_stop_recording();
-	return error;
-      }
-
-      /* -1 to account for the fact we don't time the first second */
-      expected_seconds = ( expected_time * 0.02 ) - 1;
-
-      elapsed_seconds =
-	timer_get_time_difference( &current_time, &start_time );
-
-      if( fabs( expected_seconds / elapsed_seconds - 1 ) > SPEED_TOLERANCE ) {
-
-	rzx_stop_recording();
-
-	ui_error(
-	  UI_ERROR_INFO,
-	  "emulator speed is %d%%: stopping competition mode RZX recording",
-	  (int)( 100 * ( expected_seconds / elapsed_seconds ) )
-	);
-
-      }
-    }
+    ui_error(
+      UI_ERROR_INFO,
+      "emulator speed is %d%%: stopping competition mode RZX recording",
+      (int)( current_speed )
+    );
 
   }
 
+  if( !rzx_competition_mode && settings_current.rzx_autosaves )
+    autosave_frame();
+
   return 0;
 }
 
 static int playback_frame( void )
 {
   int error, finished;
+  libspectrum_snap *snap;
 
-  error = libspectrum_rzx_playback_frame( rzx, &finished );
+  error = libspectrum_rzx_playback_frame( rzx, &finished, &snap );
   if( error ) return rzx_stop_playback( 0 );
 
   if( finished ) {
@@ -431,6 +493,17 @@ static int playback_frame( void )
     return rzx_stop_playback( 0 );
   }
 
+  /* Move the RZX sentinel back out to 79000 tstates; the addition of
+     the frame length is because everything is reduced by that in
+     event_frame() */
+  event_remove_type( sentinel_event );
+  event_add( RZX_SENTINEL_TIME + tstates, sentinel_event );
+
+  if( snap ) {
+    error = snapshot_copy_from( snap );
+    if( error ) return rzx_stop_playback( 0 );
+  }
+
   /* If we've got another frame to do, fetch the new instruction count and
      continue */
   rzx_instruction_count = libspectrum_rzx_instructions( rzx );
@@ -463,7 +536,7 @@ int rzx_store_byte( libspectrum_byte value )
         rzx_in_bytes, new_allocated * sizeof(libspectrum_byte)
       );
     if( ptr == NULL ) {
-      ui_error( UI_ERROR_ERROR, "Out of memory in rzx_store_byte\n" );
+      ui_error( UI_ERROR_ERROR, "Out of memory in rzx_store_byte" );
       return 1;
     }
 
@@ -483,3 +556,111 @@ int rzx_end( void )
 
   return 0;
 }
+
+static GSList*
+get_rollback_list( libspectrum_rzx *rzx )
+{
+  libspectrum_rzx_iterator it;
+  GSList *rollback_points;
+  size_t frames;
+
+  it = libspectrum_rzx_iterator_begin( rzx );
+  rollback_points = NULL;
+  frames = 0;
+
+  while( it ) {
+    libspectrum_rzx_block_id id = libspectrum_rzx_iterator_get_type( it );
+
+    switch( id ) {
+
+    case LIBSPECTRUM_RZX_INPUT_BLOCK:
+      frames += libspectrum_rzx_iterator_get_frames( it ); break;
+      
+    case LIBSPECTRUM_RZX_SNAPSHOT_BLOCK:
+      rollback_points = g_slist_append( rollback_points,
+					GINT_TO_POINTER( frames ) );
+      break;
+
+    default:
+      break;
+    }
+
+    it = libspectrum_rzx_iterator_next( it );
+  }
+
+  /* Add the final IRB in, if any */
+  if( frames )
+    rollback_points = g_slist_append( rollback_points,
+				      GINT_TO_POINTER( frames ) );
+  
+  return rollback_points;
+}
+
+static int
+start_after_rollback( libspectrum_snap *snap )
+{
+  int error;
+
+  error = snapshot_copy_from( snap );
+  if( error ) return error;
+
+  libspectrum_rzx_start_input( rzx, tstates );
+
+  error = counter_reset();
+  if( error ) return error;
+
+  return 0;
+}
+
+int
+rzx_rollback( void )
+{
+  libspectrum_snap *snap;
+  int error;
+
+  error = libspectrum_rzx_rollback( rzx, &snap );
+  if( error ) return error;
+
+  error = start_after_rollback( snap );
+  if( error ) return error;
+
+  return 0;
+}
+
+int
+rzx_rollback_to( void )
+{
+  GSList *rollback_points;
+  libspectrum_snap *snap;
+  int which, error;
+
+  rollback_points = get_rollback_list( rzx );
+
+  which = ui_get_rollback_point( rollback_points );
+
+  if( which == -1 ) return 1;
+
+  error = libspectrum_rzx_rollback_to( rzx, &snap, which );
+  if( error ) return error;
+
+  error = start_after_rollback( snap );
+  if( error ) return error;
+
+  return 0;
+}
+
+static void
+rzx_sentinel( libspectrum_dword tstates, int type, void *user_data )
+{
+  int error;
+
+  ui_error( UI_ERROR_WARNING, "RZX frame is longer than %u tstates",
+	    RZX_SENTINEL_TIME );
+  tstates -= RZX_SENTINEL_TIME_REDUCE;
+  z80.interrupts_enabled_at -= RZX_SENTINEL_TIME_REDUCE;
+
+  /* Add another sentinel event in case this frame continues a lot more after
+     this */
+  error = event_add( RZX_SENTINEL_TIME, sentinel_event );
+  if( error ) return;
+}
