@@ -1,7 +1,7 @@
 /* sound.h: Sound support
    Copyright (c) 2000-2004 Russell Marks, Matan Ziv-Av, Philip Kendall
 
-   $Id: sound.h,v 1.19 2004/03/29 14:30:53 pak21 Exp $
+   $Id: sound.h 2889 2007-05-26 17:45:08Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -30,6 +29,8 @@
 #include <libspectrum.h>
 
 void sound_init( const char *device );
+void sound_pause( void );
+void sound_unpause( void );
 void sound_end( void );
 void sound_ay_write( int reg, int val, libspectrum_dword now );
 void sound_ay_reset( void );
@@ -39,6 +40,7 @@ void sound_beeper( int is_tape, int on );
 extern int sound_enabled;
 extern int sound_enabled_ever;
 extern int sound_freq;
+extern int sound_framesiz;
 extern int sound_stereo;
 extern int sound_stereo_beeper;
 extern int sound_stereo_ay;
