@@ -1,7 +1,7 @@
 /* tape.c: tape handling routines
-   Copyright (c) 1999-2004 Philip Kendall, Darren Salt, Witold Filipczyk
+   Copyright (c) 1999-2008 Philip Kendall, Darren Salt, Witold Filipczyk
 
-   $Id: tape.c,v 1.106 2004/01/06 12:33:12 pak21 Exp $
+   $Id: tape.c 3714 2008-07-06 18:10:29Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
@@ -33,8 +32,11 @@
 
 #include <libspectrum.h>
 
+#include "debugger/debugger.h"
+#include "disk/beta.h"
 #include "event.h"
 #include "fuse.h"
+#include "loader.h"
 #include "machine.h"
 #include "memory.h"
 #include "scld.h"
@@ -43,7 +45,8 @@
 #include "settings.h"
 #include "snapshot.h"
 #include "tape.h"
-#include "trdos.h"
+#include "timer/timer.h"
+#include "ula.h"
 #include "ui/ui.h"
 #include "utils.h"
 #include "z80/z80.h"
@@ -56,26 +59,54 @@ static libspectrum_tape *tape;
 int tape_modified;
 
 /* Is the emulated tape deck playing? */
-static int tape_playing;
+int tape_playing;
+
+/* Was the tape playing started automatically? */
+static int tape_autoplay;
 
 /* Is there a high input to the EAR socket? */
 int tape_microphone;
 
+/* Debugger events */
+static const char *event_type_string = "tape";
+
+static const char *play_event_detail_string = "play",
+  *stop_event_detail_string = "stop";
+static int play_event, stop_event = -1;
+
+/* Spectrum events */
+int tape_edge_event;
+static int record_event;
+
 /* Function prototypes */
 
 static int tape_autoload( libspectrum_machine hardware );
 static int trap_load_block( libspectrum_tape_block *block );
-int trap_check_rom( void );
-static void make_name( char *name, const char *data );
+static int tape_play( int autoplay );
+static int trap_check_rom( void );
+static void make_name( unsigned char *name, const unsigned char *data );
+static void
+tape_event_record_sample( libspectrum_dword last_tstates, int type,
+			  void *user_data );
 
-/* Function defintions */
+/* Function definitions */
 
 int tape_init( void )
 {
-  libspectrum_error error;
+  tape = libspectrum_tape_alloc();
 
-  error = libspectrum_tape_alloc( &tape );
-  if( !tape ) return error;
+  play_event = debugger_event_register( event_type_string,
+					play_event_detail_string );
+  stop_event = debugger_event_register( event_type_string,
+					stop_event_detail_string );
+  if( play_event == -1 || stop_event == -1 ) return 1;
+
+  tape_edge_event = event_register( tape_next_edge, "Tape edge" );
+  if( tape_edge_event == -1 ) return 1;
+
+  record_event = event_register( tape_event_record_sample,
+				 "Tape sample record" );
+  if( record_event == -1 ) return 1;
 
   tape_modified = 0;
 
@@ -87,13 +118,6 @@ int tape_init( void )
   return 0;
 }
 
-/* Open a tape using the current option for whether to autoload or not */
-int
-tape_open_default_autoload( const char *filename )
-{
-  return tape_open( filename, settings_current.auto_load );
-}
-
 int tape_open( const char *filename, int autoload )
 {
   utils_file file;
@@ -126,7 +150,7 @@ tape_read_buffer( unsigned char *buffer, size_t length, libspectrum_id_t type,
   if( error ) return error;
 
   tape_modified = 0;
-  ui_tape_browser_update();
+  ui_tape_browser_update( UI_TAPE_BROWSER_NEW_TAPE, NULL );
 
   if( autoload ) {
     error = tape_autoload( machine_current->machine );
@@ -143,6 +167,7 @@ tape_autoload( libspectrum_machine hardware )
   int error; const char *id; int fd;
   char filename[80];
   utils_file snap;
+  libspectrum_id_t type;
 
   id = machine_get_id( hardware );
   if( !id ) {
@@ -150,8 +175,17 @@ tape_autoload( libspectrum_machine hardware )
     return 1;
   }
 
-  snprintf( filename, 80, "tape_%s.z80", id );
+  /* Look for an autoload snap. Try .szx first, then .z80 */
+  type = LIBSPECTRUM_ID_SNAPSHOT_SZX;
+  snprintf( filename, 80, "tape_%s.szx", id );
   fd = utils_find_auxiliary_file( filename, UTILS_AUXILIARY_LIB );
+  if( fd == -1 ) {
+  type = LIBSPECTRUM_ID_SNAPSHOT_Z80;
+    snprintf( filename, 80, "tape_%s.z80", id );
+    fd = utils_find_auxiliary_file( filename, UTILS_AUXILIARY_LIB );
+  }
+    
+  /* If we couldn't find either, give up */
   if( fd == -1 ) {
     ui_error( UI_ERROR_ERROR,
 	      "Couldn't find autoload snap for machine type '%s'", id );
@@ -161,8 +195,7 @@ tape_autoload( libspectrum_machine hardware )
   error = utils_read_fd( fd, filename, &snap );
   if( error ) return error;
 
-  error = snapshot_read_buffer( snap.buffer, snap.length,
-				LIBSPECTRUM_ID_SNAPSHOT_Z80 );
+  error = snapshot_read_buffer( snap.buffer, snap.length, type );
   if( error ) { utils_close_file( &snap ); return error; }
 
   if( utils_close_file( &snap ) ) {
@@ -183,7 +216,8 @@ int tape_close( void )
   /* If the tape has been modified, check if we want to do this */
   if( tape_modified ) {
 
-    confirm = ui_confirm_save( "Tape has been modified" );
+    confirm =
+      ui_confirm_save( "Tape has been modified.\nDo you want to save it?" );
     switch( confirm ) {
 
     case UI_CONFIRM_SAVE_SAVE:
@@ -207,7 +241,7 @@ int tape_close( void )
   if( error ) return error;
 
   tape_modified = 0;
-  ui_tape_browser_update();
+  ui_tape_browser_update( UI_TAPE_BROWSER_NEW_TAPE, NULL );
 
   return 0;
 }
@@ -220,7 +254,7 @@ tape_select_block( size_t n )
 
   error = tape_select_block_no_update( n ); if( error ) return error;
 
-  ui_tape_browser_update();
+  ui_tape_browser_update( UI_TAPE_BROWSER_SELECT_BLOCK, NULL );
 
   return 0;
 }
@@ -250,24 +284,43 @@ tape_get_current_block( void )
 /* Write the current in-memory tape file out to disk */
 int tape_write( const char* filename )
 {
+  libspectrum_id_t type;
+  libspectrum_class_t class;
   libspectrum_byte *buffer; size_t length;
 
   int error;
 
+  /* Work out what sort of file we want from the filename; default to
+     .tzx if we couldn't guess */
+  error = libspectrum_identify_file_with_class( &type, &class, filename, NULL,
+						0 );
+  if( error ) return error;
+
+  if( class != LIBSPECTRUM_CLASS_TAPE || type == LIBSPECTRUM_ID_UNKNOWN )
+    type = LIBSPECTRUM_ID_TAPE_TZX;
+
   length = 0;
-  error = libspectrum_tzx_write( &buffer, &length, tape );
+
+  error = libspectrum_tape_write( &buffer, &length, tape, type );
   if( error != LIBSPECTRUM_ERROR_NONE ) return error;
 
   error = utils_write_file( filename, buffer, length );
   if( error ) { free( buffer ); return error; }
 
   tape_modified = 0;
-  ui_tape_browser_update();
+  ui_tape_browser_update( UI_TAPE_BROWSER_MODIFIED, NULL );
+
+  free( buffer );
 
   return 0;
 
 }
 
+int tape_can_autoload( void )
+{
+  return( settings_current.auto_load && !memory_custom_rom() );
+}
+
 /* Load the next tape block into memory; returns 0 if a block was
    loaded (even if it had an tape loading error or equivalent) or
    non-zero if there was an error at the emulator level, or tape traps
@@ -278,7 +331,7 @@ int tape_load_trap( void )
   int error;
 
   /* Do nothing if tape traps aren't active, or the tape is already playing */
-  if( ! settings_current.tape_traps || tape_playing ) return 2;
+  if( !settings_current.tape_traps || tape_playing ) return 2;
 
   /* Do nothing if we're not in the correct ROM */
   if( ! trap_check_rom() ) return 3;
@@ -288,19 +341,31 @@ int tape_load_trap( void )
 
   block = libspectrum_tape_current_block( tape );
 
-  /* If this block isn't a ROM loader, start it playing
-     Then return with `error' so that we actually do whichever instruction
-     it was that caused the trap to hit */
-  if( libspectrum_tape_block_type( block ) != LIBSPECTRUM_TAPE_BLOCK_ROM ) {
-
-    error = tape_play();
-    if( error ) return 3;
+  /* Skip over any meta-data blocks */
+  while( libspectrum_tape_block_metadata( block ) ) {
+    block = libspectrum_tape_select_next_block( tape );
+    if( !block ) return 1;
+  }
+  
+  /* If this block isn't a ROM loader, start the block playing. After
+     that, return with `error' so that we actually do whichever
+     instruction it was that caused the trap to hit */
+  if( libspectrum_tape_block_type( block ) != LIBSPECTRUM_TAPE_BLOCK_ROM ||
+      libspectrum_tape_state( tape ) != LIBSPECTRUM_TAPE_STATE_PILOT ) {
+    tape_play( 1 );
+    return -1;
+  }
 
+  /* We don't properly handle the case of partial loading, so don't run
+     the traps in that situation */
+  if( libspectrum_tape_block_data_length( block ) != DE + 2 ) {
+    tape_play( 1 );
     return -1;
   }
 
   /* All returns made via the RET at #05E2, except on Timex 2068 at #0136 */
-  if ( machine_current->machine == LIBSPECTRUM_MACHINE_TC2068 ) {
+  if ( machine_current->machine == LIBSPECTRUM_MACHINE_TC2068 ||
+       machine_current->machine == LIBSPECTRUM_MACHINE_TS2068 ) {
     PC = 0x0136;
   } else {
     PC = 0x05e2;
@@ -318,74 +383,109 @@ int tape_load_trap( void )
     next_block = libspectrum_tape_select_next_block( tape );
     if( !next_block ) return 1;
 
-    ui_tape_browser_update();
+    ui_tape_browser_update( UI_TAPE_BROWSER_SELECT_BLOCK, NULL );
 
     return 0;
   }
 
   /* If the next block isn't a ROM block, set ourselves up such that the
      next thing to occur is the pause at the end of the current block */
-  libspectrum_tape_block_set_state( block, LIBSPECTRUM_TAPE_STATE_PAUSE );
-
-  /* And start the tape playing */
-  error = tape_play();
-  /* On error, still return without error as we did sucessfully do
-     the tape trap, and so don't want to do the trigger instruction */
-  if( error ) return 0;
+  libspectrum_tape_set_state( tape, LIBSPECTRUM_TAPE_STATE_PAUSE );
 
   return 0;
-
 }
 
 static int
 trap_load_block( libspectrum_tape_block *block )
 {
   libspectrum_byte parity, *data;
-  int loading, i;
+  int i = 0, length, read;
 
-  /* If the block's too short, give up and go home (with carry reset
-     to indicate error */
-  if( libspectrum_tape_block_data_length( block ) < DE ) { 
-    F = ( F & ~FLAG_C );
-    return 0;
-  }
+  /* On exit:
+   *  A = calculated parity byte if parity checked, else 0 (CHECKME)
+   *  F : if parity checked, all flags are modified
+   *      else carry only is modified (FIXME)
+   *  B = 0xB0 (success) or 0x00 (failure)
+   *  C = 0x01 (confirmed), 0x21, 0xFE or 0xDE (CHECKME)
+   * DE : decremented by number of bytes loaded or verified
+   *  H = calculated parity byte or undefined
+   *  L = last byte read, or 1 if none
+   * IX : incremented by number of bytes loaded or verified
+   * A' = unchanged on error + no flag byte, else 0x01
+   * F' = 0x01      on error + no flag byte, else 0x45
+   *  R = no point in altering it :-)
+   * Other registers unchanged.
+   */
 
   data = libspectrum_tape_block_data( block );
-  parity = *data;
+  length = libspectrum_tape_block_data_length( block );
 
-  /* If the flag byte (stored in A') does not match, reset carry and return */
-  if( *data++ != A_ ) {
-    F = ( F & ~FLAG_C );
+  /* Number of bytes to load or verify */
+  read = length - 1;
+  if( read > DE )
+    read = DE;
+
+  /* If there's no data in the block (!), set L then error exit.
+   * We don't need to alter H, IX or DE here */
+  if( !length ) {
+    L = F_ = 1;
+    F &= ~FLAG_C;
     return 0;
   }
 
-  /* Loading or verifying determined by the carry flag of F' */
-  loading = ( F_ & FLAG_C );
+  i = A_; /* i = A' (flag byte) */
+  AF_ = 0x0145;
+  A = 0;
 
-  if( loading ) {
-    for( i=0; i<DE; i++ ) {
-      writebyte_internal( IX+i, *data );
-      parity ^= *data++;
+  /* Initialise the parity check and L to the block ID byte */
+  L = parity = *data++;
+
+  /* If the block ID byte != the flag byte, clear carry and return */
+  if( parity != i )
+    goto error_ret;
+
+  /* Now set L to the *last* byte in the block */
+  L = data[read - 1];
+
+  /* Loading or verifying determined by the carry flag of F' */
+  if( F_ & FLAG_C ) {
+    for( i = 0; i < read; i++ ) {
+      parity ^= data[i];
+      writebyte_internal( IX+i, data[i] );
     }
   } else {		/* verifying */
-    for( i=0; i<DE; i++) {
-      parity ^= *data;
-      if( *data++ != readbyte_internal(IX+i) ) {
-	F = ( F & ~FLAG_C );
-	return 0;
+    for( i = 0; i < read; i++ ) {
+      parity ^= data[i];
+      if( data[i] != readbyte_internal(IX+i) ) {
+        /* Verification failure */
+        L = data[i];
+	goto error_ret;
       }
     }
   }
 
-  /* If the parity byte does not match, reset carry and return */
-  if( *data++ != parity ) {
-    F = ( F & ~FLAG_C );
-    return 0;
+  /* At this point, i == number of bytes actually read or verified */
+
+  /* If |DE| bytes have been read and there's more data, do the parity check */
+  if( DE == i && read + 1 < length ) {
+    parity ^= data[read];
+    A = parity;
+    CP( 1 ); /* parity check is successful if A==0 */
+    B = 0xB0;
+  } else {
+    /* Failure to read first bit of the next byte (ref. 48K ROM, 0x5EC) */
+    B = 255;
+    L = 1;
+    INC( B );
+error_ret:
+    F &= ~FLAG_C;
   }
 
-  /* Else return with carry set */
-  F |= FLAG_C;
-
+  /* At this point, AF, AF', B and L are already modified */
+  C = 1;
+  H = parity;
+  DE -= i;
+  IX += i;
   return 0;
 }
 
@@ -398,17 +498,15 @@ int tape_save_trap( void )
   libspectrum_byte parity, *data;
   size_t length;
 
-  int i; libspectrum_error error;
+  int i;
 
   /* Do nothing if tape traps aren't active */
-  if( ! settings_current.tape_traps ) return 2;
+  if( !settings_current.tape_traps || tape_recording ) return 2;
 
   /* Check we're in the right ROM */
   if( ! trap_check_rom() ) return 3;
 
-  /* Get a new block to store this data in */
-  error = libspectrum_tape_block_alloc( &block, LIBSPECTRUM_TAPE_BLOCK_ROM );
-  if( error ) return error;
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_ROM );
   
   /* The +2 here is for the flag and parity bytes */
   length = DE + 2;
@@ -434,15 +532,14 @@ int tape_save_trap( void )
   /* Give a 1 second pause after this block */
   libspectrum_tape_block_set_pause( block, 1000 );
 
-  /* Add the block to the current tape file */
-  error = libspectrum_tape_append_block( tape, block );
-  if( error ) return error;
+  libspectrum_tape_append_block( tape, block );
 
   tape_modified = 1;
-  ui_tape_browser_update();
+  ui_tape_browser_update( UI_TAPE_BROWSER_NEW_BLOCK, block );
 
   /* And then return via the RET at #053E, except on Timex 2068 at #00E4 */
-  if ( machine_current->machine == LIBSPECTRUM_MACHINE_TC2068 ) {
+  if ( machine_current->machine == LIBSPECTRUM_MACHINE_TC2068 ||
+       machine_current->machine == LIBSPECTRUM_MACHINE_TS2068 ) {
     PC = 0x00e4;
   } else {
     PC = 0x053e;
@@ -453,8 +550,15 @@ int tape_save_trap( void )
 }
 
 /* Check whether we're actually in the right ROM when a tape trap hit */
-int trap_check_rom( void )
+static int
+trap_check_rom( void )
 {
+  if( plusd_available && plusd_active )
+    return 0;		/* +D must not be active */
+
+  if( memory_custom_rom() )
+    return 0;           /* and we can't be using a custom ROM */
+
   switch( machine_current->machine ) {
   case LIBSPECTRUM_MACHINE_16:
   case LIBSPECTRUM_MACHINE_48:
@@ -462,58 +566,55 @@ int trap_check_rom( void )
     return 1;		/* Always OK here */
 
   case LIBSPECTRUM_MACHINE_TC2068:
+  case LIBSPECTRUM_MACHINE_TS2068:
     /* OK if we're in the EXROM (location of the tape routines) */
-    return( memory_map[0].page == timex_exrom[0].page );
+    return( memory_map_read[0].bank == MEMORY_BANK_EXROM );
 
   case LIBSPECTRUM_MACHINE_128:
   case LIBSPECTRUM_MACHINE_PLUS2:
+  case LIBSPECTRUM_MACHINE_SE:
     /* OK if we're in ROM 1 */
     return( machine_current->ram.current_rom == 1 );
 
   case LIBSPECTRUM_MACHINE_PLUS2A:
   case LIBSPECTRUM_MACHINE_PLUS3:
+  case LIBSPECTRUM_MACHINE_PLUS3E:
     /* OK if we're not in a 64Kb RAM configuration and we're in
        ROM 3 */
     return( ! machine_current->ram.special &&
 	    machine_current->ram.current_rom == 3 );
 
   case LIBSPECTRUM_MACHINE_PENT:
-    /* OK if we're in ROM 1  and TRDOS is not active */
-    return( machine_current->ram.current_rom == 1 && !trdos_active );
+  case LIBSPECTRUM_MACHINE_PENT512:
+  case LIBSPECTRUM_MACHINE_PENT1024:
+  case LIBSPECTRUM_MACHINE_SCORP:
+    /* OK if we're in ROM 1 and the Beta disk interface is not active */
+    return( machine_current->ram.current_rom == 1 && !beta_active );
+
+  case LIBSPECTRUM_MACHINE_UNKNOWN:	/* should never happen */
+    ui_error( UI_ERROR_ERROR,
+	      "trap_check_rom: machine type is LIBSPECTRUM_MACHINE_UNKNOWN" );
+    fuse_abort();
 
-  default:
-    ui_error( UI_ERROR_ERROR, "Unknown machine type %d\n",
-	      machine_current->machine );
-    abort();
   }
 
-  ui_error( UI_ERROR_ERROR, "Impossible machine type %d",
+  ui_error( UI_ERROR_ERROR, "trap_check_rom: unknown machine type %d",
 	    machine_current->machine );
   fuse_abort();
-  return 0; /* Keep gcc happy */
 }
 
-int tape_play( void )
+static int
+tape_play( int autoplay )
 {
   libspectrum_tape_block* block;
 
-  int error;
-
   if( !libspectrum_tape_present( tape ) ) return 1;
   
   block = libspectrum_tape_current_block( tape );
 
-  /* If tape traps are active and the current block is a ROM block, do
-     nothing, _unless_ the ROM block has already reached the pause at
-     its end which (hopefully) means we're in the magic state involving
-     starting slow loading whilst tape traps are active */
-  if( settings_current.tape_traps &&
-      libspectrum_tape_block_type( block ) == LIBSPECTRUM_TAPE_BLOCK_ROM &&
-      libspectrum_tape_block_state( block ) != LIBSPECTRUM_TAPE_STATE_PAUSE )
-    return 0;
-
   /* Otherwise, start the tape going */
   tape_playing = 1;
+  tape_autoplay = autoplay;
   tape_microphone = 0;
 
   /* Update the status bar */
@@ -523,47 +624,237 @@ int tape_play( void )
   if( ( !( machine_current->timex ) ) && settings_current.sound_load )
     sound_beeper( 1, tape_microphone );
 
-  error = tape_next_edge( tstates ); if( error ) return error;
+  /* If we're fastloading, turn sound off */
+  if( settings_current.fastload ) sound_pause();
+
+  loader_tape_play();
+
+  tape_next_edge( tstates, 0, NULL );
+
+  debugger_event( play_event );
 
   return 0;
 }
 
-int tape_toggle_play( void )
+int
+tape_do_play( int autoplay )
+{
+  if( !tape_playing ) {
+    return tape_play( autoplay );
+  } else {
+    return 0;
+  }
+}
+
+int tape_toggle_play( int autoplay )
 {
   if( tape_playing ) {
     return tape_stop();
   } else {
-    return tape_play();
+    return tape_play( autoplay );
   }
 }
 
 int tape_stop( void )
 {
-  tape_playing = 0;
-  ui_statusbar_update( UI_STATUSBAR_ITEM_TAPE, UI_STATUSBAR_STATE_INACTIVE );
+  if( tape_playing ) {
+
+    tape_playing = 0;
+    ui_statusbar_update( UI_STATUSBAR_ITEM_TAPE, UI_STATUSBAR_STATE_INACTIVE );
+    loader_tape_stop();
+
+    /* If we were fastloading, sound was off, so turn it back on, and
+       reset the speed counter */
+    if( settings_current.fastload ) {
+      sound_unpause();
+      timer_estimate_reset();
+    }
+
+    event_remove_type( tape_edge_event );
+  }
+
+  if( stop_event != -1 ) debugger_event( stop_event );
+
   return 0;
 }
 
 int
-tape_next_edge( libspectrum_dword last_tstates )
+tape_is_playing( void )
 {
-  int error; libspectrum_error libspec_error;
+  return tape_playing;
+}
+
+int
+tape_present( void )
+{
+  return libspectrum_tape_present( tape );
+}
+
+typedef struct
+{
+  libspectrum_byte *tape_buffer;
+  libspectrum_dword tape_buffer_size;
+  libspectrum_dword tape_buffer_used;
+  int tstates_per_sample;
+  int last_level;
+  int last_level_count;
+} tape_rec_state;
+
+int tape_recording = 0;
+
+static tape_rec_state rec_state;
+
+int
+tape_record_start( void )
+{
+  int error;
+
+  /* sample rate will be 44.1KHz */
+  rec_state.tstates_per_sample =
+    machine_current->timings.processor_speed/44100;
+
+  rec_state.tape_buffer_size = 8192;
+  rec_state.tape_buffer = malloc(rec_state.tape_buffer_size);
+  rec_state.tape_buffer_used = 0;
+
+  /* start scheduling events that record into a buffer that we
+     start allocating here */
+  error = event_add( tstates + rec_state.tstates_per_sample,
+                     record_event );
+  if( error ) return error;
+
+  rec_state.last_level = ula_tape_level();
+  rec_state.last_level_count = 1;
+
+  tape_recording = 1;
+
+  /* Also want to disable other tape actions */
+  ui_menu_activate( UI_MENU_ITEM_TAPE_RECORDING, 1 );
+
+  return 0;
+}
+
+static int
+write_rec_buffer( libspectrum_byte *tape_buffer,
+                  libspectrum_dword tape_buffer_used,
+                  int last_level_count )
+{
+  if( last_level_count <= 0xff ) {
+    tape_buffer[ tape_buffer_used++ ] = last_level_count;
+  } else {
+    tape_buffer[ tape_buffer_used++ ] = 0;
+    tape_buffer[ tape_buffer_used++ ] = ( last_level_count & 0x000000ff )      ;
+    tape_buffer[ tape_buffer_used++ ] = ( last_level_count & 0x0000ff00 ) >>  8;
+    tape_buffer[ tape_buffer_used++ ] = ( last_level_count & 0x00ff0000 ) >> 16;
+    tape_buffer[ tape_buffer_used++ ] = ( last_level_count & 0xff000000 ) >> 24;
+  }
+
+  return tape_buffer_used;
+}
+
+void
+tape_event_record_sample( libspectrum_dword last_tstates, int type,
+			  void *user_data )
+{
+  int error;
+
+  if( rec_state.last_level != (ula_tape_level()) ) {
+    /* put a sample into the recording buffer */
+    rec_state.tape_buffer_used =
+      write_rec_buffer( rec_state.tape_buffer,
+                        rec_state.tape_buffer_used,
+                        rec_state.last_level_count );
+
+    rec_state.last_level_count = 0;
+    rec_state.last_level = ula_tape_level();
+    /* make sure we can still fit a dword and a flag byte in the buffer */
+    if( rec_state.tape_buffer_used+5 >= rec_state.tape_buffer_size ) {
+      rec_state.tape_buffer_size = rec_state.tape_buffer_size*2;
+      rec_state.tape_buffer = realloc( rec_state.tape_buffer,
+                                       rec_state.tape_buffer_size );
+    }
+  }
+
+  rec_state.last_level_count++;
+
+  /* schedule next timer */
+  error = event_add( last_tstates + rec_state.tstates_per_sample,
+                     record_event );
+  if( error ) {
+    ui_error( UI_ERROR_ERROR,
+              "tape_event_record_sample: error scheduling next event" );
+  }
+}
+
+int
+tape_record_stop( void )
+{
+  libspectrum_tape_block* block;
+  int error;
+
+  /* put last sample into the recording buffer */
+  rec_state.tape_buffer_used = write_rec_buffer( rec_state.tape_buffer,
+                                                 rec_state.tape_buffer_used,
+                                                 rec_state.last_level_count );
+
+  /* stop scheduling events and turn buffer into a block and
+     pop into the current tape */
+  error = event_remove_type( record_event );
+  if( error ) return error;
+
+  block = libspectrum_tape_block_alloc( LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE );
+
+  libspectrum_tape_block_set_scale( block, rec_state.tstates_per_sample );
+  libspectrum_tape_block_set_data_length( block, rec_state.tape_buffer_used );
+  libspectrum_tape_block_set_data( block, rec_state.tape_buffer );
+
+  libspectrum_tape_append_block( tape, block );
+
+  rec_state.tape_buffer = NULL;
+  rec_state.tape_buffer_size = 0;
+  rec_state.tape_buffer_used = 0;
+
+  tape_modified = 1;
+  ui_tape_browser_update( UI_TAPE_BROWSER_NEW_BLOCK, block );
+
+  tape_recording = 0;
+
+  /* Also want to reenable other tape actions */
+  ui_menu_activate( UI_MENU_ITEM_TAPE_RECORDING, 0 );
+
+  return 0;
+}
+
+void
+tape_next_edge( libspectrum_dword last_tstates, int type, void *user_data )
+{
+  libspectrum_error libspec_error;
   libspectrum_tape_block *block;
 
   libspectrum_dword edge_tstates;
   int flags;
 
   /* If the tape's not playing, just return */
-  if( ! tape_playing ) return 0;
+  if( ! tape_playing ) return;
 
   /* Get the time until the next edge */
   libspec_error = libspectrum_tape_get_next_edge( &edge_tstates, &flags,
 						  tape );
-  if( libspec_error != LIBSPECTRUM_ERROR_NONE ) return libspec_error;
+  if( libspec_error != LIBSPECTRUM_ERROR_NONE ) return;
 
   /* Invert the microphone state */
   if( edge_tstates || ( flags & LIBSPECTRUM_TAPE_FLAGS_STOP ) ) {
-    tape_microphone = !tape_microphone;
+
+    if( flags & LIBSPECTRUM_TAPE_FLAGS_NO_EDGE ) {
+      /* Do nothing */
+    } else if( flags & LIBSPECTRUM_TAPE_FLAGS_LEVEL_LOW ) {
+      tape_microphone = 0;
+    } else if( flags & LIBSPECTRUM_TAPE_FLAGS_LEVEL_HIGH ) {
+      tape_microphone = 1;
+    } else {
+      tape_microphone = !tape_microphone;
+    }
+
     /* Timex machines have no loading noise */
     if( !machine_current->timex && settings_current.sound_load )
       sound_beeper( 1, tape_microphone );
@@ -580,23 +871,24 @@ tape_next_edge( libspectrum_dword last_tstates )
       )
     )
   {
-    error = tape_stop(); if( error ) return error;
-    return 0;
+    tape_stop();
+    return;
   }
 
   /* If that was the end of a block, update the browser */
   if( flags & LIBSPECTRUM_TAPE_FLAGS_BLOCK ) {
 
-    ui_tape_browser_update();
+    ui_tape_browser_update( UI_TAPE_BROWSER_SELECT_BLOCK, NULL );
 
-    /* If tape traps are active _and_ the new block is a ROM loader,
-     return without putting another event into the queue */
+    /* If the tape was started automatically, tape traps are active
+       and the new block is a ROM loader, stop the tape and return
+       without putting another event into the queue */
     block = libspectrum_tape_current_block( tape );
-    if( settings_current.tape_traps &&
+    if( tape_autoplay && settings_current.tape_traps &&
 	libspectrum_tape_block_type( block ) == LIBSPECTRUM_TAPE_BLOCK_ROM
       ) {
-      error = tape_stop(); if( error ) return error;
-      return 0;
+      tape_stop();
+      return;
     }
   }
 
@@ -604,10 +896,10 @@ tape_next_edge( libspectrum_dword last_tstates )
      should occur 'edge_tstates' after the last edge, not after the
      current time (these will be slightly different as we only process
      events between instructions). */
-  error = event_add( last_tstates + edge_tstates, EVENT_TYPE_EDGE );
-  if( error ) return error;
+  if( event_add( last_tstates + edge_tstates, tape_edge_event ) ) return;
 
-  return 0;
+  /* Store length flags for acceleration purposes */
+  loader_set_acceleration_flags( flags );
 }
 
 /* Call a user-supplied function for every block in the current tape */
@@ -632,9 +924,11 @@ tape_block_details( char *buffer, size_t length,
 		    libspectrum_tape_block *block )
 {
   libspectrum_byte *data;
-  const char *type; char name[11];
+  const char *type; unsigned char name[11];
   int offset;
 
+  buffer[0] = '\0';
+
   switch( libspectrum_tape_block_type( block ) ) {
 
   case LIBSPECTRUM_TAPE_BLOCK_ROM:
@@ -714,13 +1008,21 @@ tape_block_details( char *buffer, size_t length,
 	      (unsigned long)libspectrum_tape_block_count( block ) );
     break;
 
+  case LIBSPECTRUM_TAPE_BLOCK_GENERALISED_DATA:
+    snprintf( buffer, length, "%lu data symbols",
+	      (unsigned long)libspectrum_tape_generalised_data_symbol_table_symbols_in_block( libspectrum_tape_block_data_table( block ) ) );
+    break;
+
+  case LIBSPECTRUM_TAPE_BLOCK_RLE_PULSE:
+    /* Could do something better with this one */
+    break;
+
   case LIBSPECTRUM_TAPE_BLOCK_GROUP_END:
   case LIBSPECTRUM_TAPE_BLOCK_LOOP_END:
   case LIBSPECTRUM_TAPE_BLOCK_STOP48:
   case LIBSPECTRUM_TAPE_BLOCK_ARCHIVE_INFO:
   case LIBSPECTRUM_TAPE_BLOCK_HARDWARE:
-  default:
-    buffer[0] = '\0';
+  case LIBSPECTRUM_TAPE_BLOCK_CONCAT:
     break;
 
   }
@@ -729,7 +1031,7 @@ tape_block_details( char *buffer, size_t length,
 }
 
 static void
-make_name( char *name, const char *data )
+make_name( unsigned char *name, const unsigned char *data )
 {
   size_t i;
 
@@ -743,4 +1045,3 @@ make_name( char *name, const char *data )
 
   *name = '\0';
 }
-
