@@ -1,7 +1,7 @@
 /* fbui.c: Routines for dealing with the linux fbdev user interface
-   Copyright (c) 2000-2003 Philip Kendall, Matan Ziv-Av, Witold Filipczyk
+   Copyright (c) 2000-2004 Philip Kendall, Matan Ziv-Av, Witold Filipczyk
 
-   $Id: fbui.c,v 1.15 2003/04/13 09:47:36 pak21 Exp $
+   $Id: fbui.c 3096 2007-08-06 09:20:34Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,42 +13,68 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
-#ifdef UI_FB			/* Use this iff we're using fbdev */
-
+#include <errno.h>
 #include <stdlib.h>
+#include <signal.h>
+#include <string.h>
 
 #include "fbdisplay.h"
 #include "fbkeyboard.h"
+#include "fbmouse.h"
 #include "ui/ui.h"
 #include "ui/uidisplay.h"
 
+static void end_handler( int signo );
 static void fb_end( void );
 
 int
 ui_init( int *argc, char ***argv )
 {
+  struct sigaction handler;
   int error;
 
-  atexit( fb_end );
+  error = atexit( fb_end );
+  if( error ) {
+    ui_error( UI_ERROR_ERROR, "ui_init: couldn't set atexit function" );
+    return 1;
+  }
+  
+  handler.sa_handler = end_handler;
+
+  error = sigaction( SIGTERM, &handler, NULL );
+  if( error ) {
+    ui_error( UI_ERROR_ERROR, "ui_init: couldn't set SIGTERM handler: %s",
+	      strerror( errno ) );
+    return 1;
+  }
+
+  error = sigaction( SIGHUP, &handler, NULL );
+  if( error ) {
+    ui_error( UI_ERROR_ERROR, "ui_init: couldn't set SIGHUP handler: %s",
+	      strerror( errno ) );
+    return 1;
+  }
 
   error = fbkeyboard_init();
-  if(error) return error;
+  if( error ) return error;
+
+  error = fbmouse_init();
+  if( error ) return error;
 
   error = fbdisplay_init();
-  if(error) return error;
+  if( error ) return error;
 
   return 0;
 }
@@ -56,6 +82,7 @@ ui_init( int *argc, char ***argv )
 int ui_event()
 {
   keyboard_update();
+  mouse_update();
   return 0;
 }
 
@@ -70,11 +97,15 @@ int ui_end(void)
   return 0;
 }
 
+static void
+end_handler( int signo )
+{
+  fb_end();
+}
+
 static void
 fb_end( void )
 {
   fbkeyboard_end();
   uidisplay_end();
 }
-
-#endif				/* #ifdef UI_FB */
