@@ -1,7 +1,7 @@
 /* gtkdisplay.c: GTK+ routines for dealing with the Speccy screen
-   Copyright (c) 2000-2003 Philip Kendall
+   Copyright (c) 2000-2005 Philip Kendall
 
-   $Id: gtkdisplay.c,v 1.52.2.2 2004/02/09 12:25:48 pak21 Exp $
+   $Id: gtkdisplay.c 3612 2008-05-02 12:56:05Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,21 +13,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
-#ifdef UI_GTK		/* Use this file iff we're using GTK+ */
-
 #include <stddef.h>
 #include <stdio.h>
 #include <string.h>
@@ -51,6 +48,12 @@ int image_scale;
 /* The height and width of a 1x1 image in pixels */
 int image_width, image_height;
 
+/* A copy of every pixel on the screen, replaceable by plotting directly into
+   rgb_image below */
+libspectrum_word
+  gtkdisplay_image[ 2 * DISPLAY_SCREEN_HEIGHT ][ DISPLAY_SCREEN_WIDTH ];
+ptrdiff_t gtkdisplay_pitch = DISPLAY_SCREEN_WIDTH * sizeof( libspectrum_word );
+
 /* An RGB image of the Spectrum screen; slightly bigger than the real
    screen to handle the smoothing filters which read around each pixel */
 static guchar rgb_image[ 4 * 2 * ( DISPLAY_SCREEN_HEIGHT + 4 ) *
@@ -63,7 +66,7 @@ static guchar scaled_image[ 4 * 3 * DISPLAY_SCREEN_HEIGHT *
 static const ptrdiff_t scaled_pitch = 4 * 1.5 * DISPLAY_SCREEN_WIDTH;
 
 /* The colour palette */
-static guchar rgb_colours[16][3] = {
+static const guchar rgb_colours[16][3] = {
 
   {   0,   0,   0 },
   {   0,   0, 192 },
@@ -93,15 +96,14 @@ static int gtkdisplay_current_size=1;
 
 static int init_colours( void );
 static void gtkdisplay_area(int x, int y, int width, int height);
-static int gtkdisplay_configure_notify( int width );
 static int register_scalers( void );
 
 /* Callbacks */
 
 static gint gtkdisplay_expose(GtkWidget *widget, GdkEvent *event,
 			      gpointer data);
-static gint gtkdisplay_configure(GtkWidget *widget, GdkEvent *event,
-				 gpointer data);
+static gint drawing_area_resize_callback( GtkWidget *widget, GdkEvent *event,
+					  gpointer data );
 
 int
 gtkdisplay_init( void )
@@ -112,7 +114,7 @@ gtkdisplay_init( void )
   gtk_signal_connect( GTK_OBJECT(gtkui_drawing_area), "expose_event", 
 		      GTK_SIGNAL_FUNC(gtkdisplay_expose), NULL);
   gtk_signal_connect( GTK_OBJECT(gtkui_drawing_area), "configure_event", 
-		      GTK_SIGNAL_FUNC(gtkdisplay_configure), NULL);
+		      GTK_SIGNAL_FUNC( drawing_area_resize_callback ), NULL);
 
   error = init_colours(); if( error ) return error;
 
@@ -134,6 +136,7 @@ init_colours( void )
 
   for( i = 0; i < 16; i++ ) {
 
+
     guchar red, green, blue, grey;
 
     red   = rgb_colours[i][0];
@@ -161,6 +164,7 @@ init_colours( void )
 }
 
 int
+
 uidisplay_init( int width, int height )
 {
   int error;
@@ -175,24 +179,23 @@ uidisplay_init( int width, int height )
   return 0;
 }
 
-static int gtkdisplay_configure_notify( int width )
+static int
+drawing_area_resize( int width, int height )
 {
   int size, error;
 
   size = width / DISPLAY_ASPECT_WIDTH;
+  if( size > height / DISPLAY_SCREEN_HEIGHT )
+    size = height / DISPLAY_SCREEN_HEIGHT;
 
-  /* If we're the same size as before, nothing special needed */
+  /* If we're the same size as before, no need to do anything else */
   if( size == gtkdisplay_current_size ) return 0;
 
-  /* Else set ourselves to the new height */
-  gtkdisplay_current_size=size;
-  gtk_drawing_area_size( GTK_DRAWING_AREA(gtkui_drawing_area),
-			 size * DISPLAY_ASPECT_WIDTH,
-			 size * DISPLAY_SCREEN_HEIGHT );
+  gtkdisplay_current_size = size;
 
   error = register_scalers(); if( error ) return error;
 
-  /* Redraw the entire screen... */
+  memset( scaled_image, 0, sizeof( scaled_image ) );
   display_refresh_all();
 
   return 0;
@@ -210,6 +213,7 @@ register_scalers( void )
     switch( image_scale ) {
     case 1:
       scaler_register( SCALER_NORMAL );
+      scaler_register( SCALER_PALTV );
       if( !scaler_is_supported( current_scaler ) )
 	scaler_select_scaler( SCALER_NORMAL );
       return 0;
@@ -232,12 +236,15 @@ register_scalers( void )
       scaler_register( SCALER_SUPER2XSAI );
       scaler_register( SCALER_SUPEREAGLE );
       scaler_register( SCALER_DOTMATRIX );
+      scaler_register( SCALER_PALTV2X );
+      scaler_register( SCALER_HQ2X );
       if( !scaler_is_supported( current_scaler ) )
 	scaler_select_scaler( SCALER_DOUBLESIZE );
       return 0;
     case 2:
       scaler_register( SCALER_NORMAL );
       scaler_register( SCALER_TIMEXTV );
+      scaler_register( SCALER_PALTV );
       if( !scaler_is_supported( current_scaler ) )
 	scaler_select_scaler( SCALER_NORMAL );
       return 0;
@@ -248,7 +255,10 @@ register_scalers( void )
     switch( image_scale ) {
     case 1:
       scaler_register( SCALER_TRIPLESIZE );
+      scaler_register( SCALER_TV3X );
       scaler_register( SCALER_ADVMAME3X );
+      scaler_register( SCALER_PALTV3X );
+      scaler_register( SCALER_HQ3X );
       if( !scaler_is_supported( current_scaler ) )
 	scaler_select_scaler( SCALER_TRIPLESIZE );
       return 0;
@@ -296,7 +306,7 @@ uidisplay_area( int x, int y, int w, int h )
     rgb = (libspectrum_dword*)( rgb_image + ( yy + 2 ) * rgb_pitch );
     rgb += x + 1;
 
-    display = &display_image[yy][x];
+    display = &gtkdisplay_image[yy][x];
 
     for( i = 0; i < w; i++, rgb++, display++ ) *rgb = palette[ *display ];
   }
@@ -322,7 +332,7 @@ static void gtkdisplay_area(int x, int y, int width, int height)
 			 scaled_pitch );
 }
 
-void
+int
 uidisplay_hotswap_gfx_mode( void )
 {
   fuse_emulation_pause();
@@ -331,6 +341,8 @@ uidisplay_hotswap_gfx_mode( void )
   display_refresh_all();
 
   fuse_emulation_unpause();
+
+  return 0;
 }
 
 int
@@ -345,6 +357,92 @@ gtkdisplay_end( void )
   return 0;
 }
 
+/* Set one pixel in the display */
+void
+uidisplay_putpixel( int x, int y, int colour )
+{
+  if( machine_current->timex ) {
+    x <<= 1; y <<= 1;
+    gtkdisplay_image[y  ][x  ] = colour;
+    gtkdisplay_image[y  ][x+1] = colour;
+    gtkdisplay_image[y+1][x  ] = colour;
+    gtkdisplay_image[y+1][x+1] = colour;
+  } else {
+    gtkdisplay_image[y][x] = colour;
+  }
+}
+
+/* Print the 8 pixels in `data' using ink colour `ink' and paper
+   colour `paper' to the screen at ( (8*x) , y ) */
+void
+uidisplay_plot8( int x, int y, libspectrum_byte data,
+                libspectrum_byte ink, libspectrum_byte paper )
+{
+  x <<= 3;
+
+  if( machine_current->timex ) {
+    int i;
+
+    x <<= 1; y <<= 1;
+    for( i=0; i<2; i++,y++ ) {
+      gtkdisplay_image[y][x+ 0] = ( data & 0x80 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 1] = ( data & 0x80 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 2] = ( data & 0x40 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 3] = ( data & 0x40 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 4] = ( data & 0x20 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 5] = ( data & 0x20 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 6] = ( data & 0x10 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 7] = ( data & 0x10 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 8] = ( data & 0x08 ) ? ink : paper;
+      gtkdisplay_image[y][x+ 9] = ( data & 0x08 ) ? ink : paper;
+      gtkdisplay_image[y][x+10] = ( data & 0x04 ) ? ink : paper;
+      gtkdisplay_image[y][x+11] = ( data & 0x04 ) ? ink : paper;
+      gtkdisplay_image[y][x+12] = ( data & 0x02 ) ? ink : paper;
+      gtkdisplay_image[y][x+13] = ( data & 0x02 ) ? ink : paper;
+      gtkdisplay_image[y][x+14] = ( data & 0x01 ) ? ink : paper;
+      gtkdisplay_image[y][x+15] = ( data & 0x01 ) ? ink : paper;
+    }
+  } else {
+    gtkdisplay_image[y][x+ 0] = ( data & 0x80 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 1] = ( data & 0x40 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 2] = ( data & 0x20 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 3] = ( data & 0x10 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 4] = ( data & 0x08 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 5] = ( data & 0x04 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 6] = ( data & 0x02 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 7] = ( data & 0x01 ) ? ink : paper;
+  }
+}
+
+/* Print the 16 pixels in `data' using ink colour `ink' and paper
+   colour `paper' to the screen at ( (16*x) , y ) */
+void
+uidisplay_plot16( int x, int y, libspectrum_word data,
+                 libspectrum_byte ink, libspectrum_byte paper )
+{
+  int i;
+  x <<= 4; y <<= 1;
+
+  for( i=0; i<2; i++,y++ ) {
+    gtkdisplay_image[y][x+ 0] = ( data & 0x8000 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 1] = ( data & 0x4000 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 2] = ( data & 0x2000 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 3] = ( data & 0x1000 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 4] = ( data & 0x0800 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 5] = ( data & 0x0400 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 6] = ( data & 0x0200 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 7] = ( data & 0x0100 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 8] = ( data & 0x0080 ) ? ink : paper;
+    gtkdisplay_image[y][x+ 9] = ( data & 0x0040 ) ? ink : paper;
+    gtkdisplay_image[y][x+10] = ( data & 0x0020 ) ? ink : paper;
+    gtkdisplay_image[y][x+11] = ( data & 0x0010 ) ? ink : paper;
+    gtkdisplay_image[y][x+12] = ( data & 0x0008 ) ? ink : paper;
+    gtkdisplay_image[y][x+13] = ( data & 0x0004 ) ? ink : paper;
+    gtkdisplay_image[y][x+14] = ( data & 0x0002 ) ? ink : paper;
+    gtkdisplay_image[y][x+15] = ( data & 0x0001 ) ? ink : paper;
+  }
+}
+
 /* Callbacks */
 
 /* Called by gtkui_drawing_area on "expose_event" */
@@ -359,11 +457,9 @@ gtkdisplay_expose( GtkWidget *widget GCC_UNUSED, GdkEvent *event,
 
 /* Called by gtkui_drawing_area on "configure_event" */
 static gint
-gtkdisplay_configure( GtkWidget *widget GCC_UNUSED, GdkEvent *event,
-		      gpointer data GCC_UNUSED )
+drawing_area_resize_callback( GtkWidget *widget GCC_UNUSED, GdkEvent *event,
+			      gpointer data GCC_UNUSED )
 {
-  gtkdisplay_configure_notify( event->configure.width );
-  return FALSE;
+  drawing_area_resize( event->configure.width, event->configure.height );
+  return TRUE;
 }
-
-#endif			/* #ifdef UI_GTK */
