@@ -1,7 +1,7 @@
 /* sdldisplay.c: Routines for dealing with the SDL display
-   Copyright (c) 2000-2004 Philip Kendall, Matan Ziv-Av, Fredrick Meunier
+   Copyright (c) 2000-2006 Philip Kendall, Matan Ziv-Av, Fredrick Meunier
 
-   $Id: sdldisplay.c,v 1.27.2.4 2004/02/16 00:54:14 pak21 Exp $
+   $Id: sdldisplay.c 3666 2008-06-10 20:43:46Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,21 +13,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
-#ifdef UI_SDL			/* Use this iff we're using SDL */
-
 #include <limits.h>
 #include <stdio.h>
 #include <string.h>
@@ -46,16 +43,18 @@
 #include "ui/uidisplay.h"
 #include "utils.h"
 #ifdef USE_WIDGET
-#include "widget/widget.h"
+#include "ui/widget/widget.h"
 #endif				/* #ifdef USE_WIDGET */
 
 SDL_Surface *sdldisplay_gc = NULL;   /* Hardware screen */
 static SDL_Surface *tmp_screen=NULL; /* Temporary screen for scalers */
 
-static SDL_Surface *red_cassette, *green_cassette = NULL;
-static SDL_Surface *red_disk, *green_disk = NULL;
+static SDL_Surface *red_cassette[2], *green_cassette[2];
+static SDL_Surface *red_mdr[2], *green_mdr[2];
+static SDL_Surface *red_disk[2], *green_disk[2];
 
-static ui_statusbar_state sdl_disk_state, sdl_tape_state;
+static ui_statusbar_state sdl_disk_state, sdl_mdr_state, sdl_tape_state;
+static int sdl_status_updated;
 
 static int tmp_screen_width;
 
@@ -89,6 +88,9 @@ static SDL_Rect updated_rects[MAX_UPDATE_RECT];
 static int num_rects = 0;
 static libspectrum_byte sdldisplay_force_full_refresh = 1;
 
+static int max_fullscreen_height;
+static int min_fullscreen_height;
+
 /* The current size of the display (in units of DISPLAY_SCREEN_*) */
 static float sdldisplay_current_size = 1;
 
@@ -119,6 +121,8 @@ init_scalers( void )
   scaler_register( SCALER_ADVMAME2X );
   scaler_register( SCALER_ADVMAME3X );
   scaler_register( SCALER_DOTMATRIX );
+  scaler_register( SCALER_PALTV );
+  scaler_register( SCALER_HQ2X );
   if( machine_current->timex ) {
     scaler_register( SCALER_HALF ); 
     scaler_register( SCALER_HALFSKIP );
@@ -126,6 +130,10 @@ init_scalers( void )
     scaler_register( SCALER_TIMEX1_5X );
   } else {
     scaler_register( SCALER_TV2X );
+    scaler_register( SCALER_TV3X );
+    scaler_register( SCALER_PALTV2X );
+    scaler_register( SCALER_PALTV3X );
+    scaler_register( SCALER_HQ3X );
   }
   
   if( scaler_is_supported( current_scaler ) ) {
@@ -135,13 +143,52 @@ init_scalers( void )
   }
 }
 
+static int
+sdl_convert_icon( SDL_Surface *source, SDL_Surface **icon, int red )
+{
+  SDL_Surface *copy;   /* Copy with altered palette */
+  int i;
+
+  SDL_Color colors[ source->format->palette->ncolors ];
+
+  copy = SDL_ConvertSurface( source, source->format, SDL_SWSURFACE );
+
+  for( i = 0; i < copy->format->palette->ncolors; i++ ) {
+    colors[i].r = red ? copy->format->palette->colors[i].r : 0;
+    colors[i].g = red ? 0 : copy->format->palette->colors[i].g;
+    colors[i].b = 0;
+  }
+
+  SDL_SetPalette( copy, SDL_LOGPAL, colors, 0, i );
+
+  icon[0] = SDL_ConvertSurface( copy, tmp_screen->format, SDL_SWSURFACE );
+
+  SDL_FreeSurface( copy );
+
+  icon[1] = SDL_CreateRGBSurface( SDL_SWSURFACE,
+                                  (icon[0]->w)<<1, (icon[0]->h)<<1,
+                                  icon[0]->format->BitsPerPixel,
+                                  icon[0]->format->Rmask,
+                                  icon[0]->format->Gmask,
+                                  icon[0]->format->Bmask,
+                                  icon[0]->format->Amask
+                                );
+
+  ( scaler_get_proc16( SCALER_DOUBLESIZE ) )(
+        (libspectrum_byte*)icon[0]->pixels,
+        icon[0]->pitch,
+        (libspectrum_byte*)icon[1]->pixels,
+        icon[1]->pitch, icon[0]->w, icon[0]->h
+      );
+
+  return 0;
+}
+
 static int
 sdl_load_status_icon( const char*filename, SDL_Surface **red, SDL_Surface **green )
 {
   char path[ PATH_MAX ];
   SDL_Surface *temp;    /* Copy of image as loaded */
-  SDL_Surface *temp2;   /* Copy with altered palette */
-  int i;
 
   if( utils_find_file_path( filename, path, UTILS_AUXILIARY_LIB ) ) {
     fprintf( stderr, "%s: Error getting path for icons\n", fuse_progname );
@@ -159,37 +206,8 @@ sdl_load_status_icon( const char*filename, SDL_Surface **red, SDL_Surface **gree
     return -1;
   }
 
-  {
-    SDL_Color colors[ temp->format->palette->ncolors ];
-    temp2 = SDL_ConvertSurface( temp, temp->format, SDL_SWSURFACE );
-    for( i = 0; i < temp2->format->palette->ncolors; i++ ) {
-      colors[i].r = temp2->format->palette->colors[i].r;
-      colors[i].g = 0;
-      colors[i].b = 0;
-    }
-
-    SDL_SetPalette( temp2, SDL_LOGPAL, colors, 0, i );
-
-    *red = SDL_ConvertSurface( temp2, tmp_screen->format, SDL_SWSURFACE );
-
-    SDL_FreeSurface( temp2 );
-  }
-
-  {
-    SDL_Color colors[ temp->format->palette->ncolors ];
-    temp2 = SDL_ConvertSurface( temp, temp->format, SDL_SWSURFACE );
-    for( i = 0; i < temp2->format->palette->ncolors; i++ ) {
-      colors[i].r = 0;
-      colors[i].g = temp2->format->palette->colors[i].g;
-      colors[i].b = 0;
-    }
-
-    SDL_SetPalette( temp2, SDL_LOGPAL, colors, 0, i );
-
-    *green = SDL_ConvertSurface( temp2, tmp_screen->format, SDL_SWSURFACE );
-
-    SDL_FreeSurface( temp2 );
-  }
+  sdl_convert_icon( temp, red, 1 );
+  sdl_convert_icon( temp, green, 0 );
 
   SDL_FreeSurface( temp );
 
@@ -199,21 +217,47 @@ sdl_load_status_icon( const char*filename, SDL_Surface **red, SDL_Surface **gree
 int
 uidisplay_init( int width, int height )
 {
+  SDL_Rect **modes;
+  int i;
+
+  /* Get available fullscreen/software modes */
+  modes=SDL_ListModes(NULL, SDL_FULLSCREEN|SDL_SWSURFACE);
+  /* Check if there are any modes available, or if our resolution is restricted
+     at all */
+  if( modes == (SDL_Rect **) 0 || modes == (SDL_Rect **) -1 ){
+    /* Just try whatever we have and see what happens */
+    max_fullscreen_height = 480;
+    min_fullscreen_height = 240;
+  } else {
+    /* Record the largest supported fullscreen software mode */
+    max_fullscreen_height = modes[0]->h;
+
+    /* Record the smallest supported fullscreen software mode */
+    for( i=0; modes[i]; ++i ) {
+      min_fullscreen_height = modes[i]->h;
+    }
+  }
+
   image_width = width;
   image_height = height;
 
   timex = machine_current->timex;
 
   init_scalers();
-  sdldisplay_load_gfx_mode();
+
+  if ( scaler_select_scaler( current_scaler ) )
+    scaler_select_scaler( SCALER_NORMAL );
+
+  if( sdldisplay_load_gfx_mode() ) return 1;
 
   SDL_WM_SetCaption( "Fuse", "Fuse" );
 
   /* We can now output error messages to our output device */
   display_ui_initialised = 1;
 
-  sdl_load_status_icon( "cassette.bmp", &red_cassette, &green_cassette );
-  sdl_load_status_icon( "plus3disk.bmp", &red_disk, &green_disk );
+  sdl_load_status_icon( "cassette.bmp", red_cassette, green_cassette );
+  sdl_load_status_icon( "microdrive.bmp", red_mdr, green_mdr );
+  sdl_load_status_icon( "plus3disk.bmp", red_disk, green_disk );
 
   return 0;
 }
@@ -241,6 +285,40 @@ sdldisplay_allocate_colours( int numColours, Uint32 *colour_values,
   return 0;
 }
 
+static void
+sdldisplay_find_best_fullscreen_scaler( void )
+{
+  static int windowed_scaler = -1;
+
+  /* Make sure we have at least more than half of the screen covered in
+     fullscreen to avoid the "postage stamp" on machines that don't support
+     320x240 anymore e.g. Mac notebooks */
+  if( settings_current.full_screen ) {
+    int i = 0;
+    while( i < SCALER_NUM &&
+           ( image_height*sdldisplay_current_size <= min_fullscreen_height/2 ||
+             image_height*sdldisplay_current_size > max_fullscreen_height ) ) {
+      if( windowed_scaler == -1) windowed_scaler = current_scaler;
+      while( !scaler_is_supported(i) ) i++;
+      scaler_select_scaler( i++ );
+      sdldisplay_current_size = scaler_get_scaling_factor( current_scaler );
+      /* if we failed to find a suitable size scaler, just use normal (what the
+         user had originally may be too big) */
+      if( image_height * sdldisplay_current_size <= min_fullscreen_height/2 ||
+          image_height * sdldisplay_current_size > max_fullscreen_height ) {
+        scaler_select_scaler( SCALER_NORMAL );
+        sdldisplay_current_size = scaler_get_scaling_factor( current_scaler );
+      }
+    }
+  } else {
+    if( windowed_scaler != -1 ) {
+      scaler_select_scaler( windowed_scaler );
+      windowed_scaler = -1;
+      sdldisplay_current_size = scaler_get_scaling_factor( current_scaler );
+    }
+  }
+}
+
 static int
 sdldisplay_load_gfx_mode( void )
 {
@@ -253,6 +331,8 @@ sdldisplay_load_gfx_mode( void )
 
   sdldisplay_current_size = scaler_get_scaling_factor( current_scaler );
 
+  sdldisplay_find_best_fullscreen_scaler();
+
   /* Create the surface that contains the scaled graphics in 16 bit mode */
   sdldisplay_gc = SDL_SetVideoMode(
     image_width * sdldisplay_current_size,
@@ -262,12 +342,15 @@ sdldisplay_load_gfx_mode( void )
                                  : SDL_SWSURFACE
   );
   if( !sdldisplay_gc ) {
-    fprintf( stderr, "%s: couldn't create gc\n", fuse_progname );
-    return 1;
+    fprintf( stderr, "%s: couldn't create SDL graphics context\n", fuse_progname );
+    fuse_abort();
   }
 
+  sdldisplay_is_full_screen =
+      settings_current.full_screen = !!(sdldisplay_gc->flags & SDL_FULLSCREEN);
+
   /* Distinguish 555 and 565 mode */
-  if( sdldisplay_gc->format->Rmask == 0x7C00 )
+  if( sdldisplay_gc->format->Gmask >> sdldisplay_gc->format->Gshift == 0x1f )
     scaler_select_bitformat( 555 );
   else
     scaler_select_bitformat( 565 );
@@ -287,7 +370,7 @@ sdldisplay_load_gfx_mode( void )
 
   if( !tmp_screen ) {
     fprintf( stderr, "%s: couldn't create tmp_screen\n", fuse_progname );
-    return 1;
+    fuse_abort();
   }
 
   sdldisplay_allocate_colours( 16, colour_values, bw_values );
@@ -298,7 +381,7 @@ sdldisplay_load_gfx_mode( void )
   return 0;
 }
 
-void
+int
 uidisplay_hotswap_gfx_mode( void )
 {
   fuse_emulation_pause();
@@ -310,36 +393,70 @@ uidisplay_hotswap_gfx_mode( void )
   }
 
   /* Setup the new GFX mode */
-  sdldisplay_load_gfx_mode();
+  if( sdldisplay_load_gfx_mode() ) return 1;
 
   /* reset palette */
   SDL_SetColors( sdldisplay_gc, colour_palette, 0, 16 );
 
   /* Mac OS X resets the state of the cursor after a switch to full screen
      mode */
-  if ( settings_current.full_screen )
+  if ( settings_current.full_screen || ui_mouse_grabbed ) {
     SDL_ShowCursor( SDL_DISABLE );
-  else
+    SDL_WarpMouse( 128, 128 );
+  } else {
     SDL_ShowCursor( SDL_ENABLE );
+  }
 
   fuse_emulation_unpause();
+
+  return 0;
+}
+
+SDL_Surface *saved = NULL;
+
+void
+uidisplay_frame_save( void )
+{
+  if( saved ) {
+    SDL_FreeSurface( saved );
+    saved = NULL;
+  }
+
+  saved = SDL_ConvertSurface( tmp_screen, tmp_screen->format,
+                              SDL_SWSURFACE );
+}
+
+void
+uidisplay_frame_restore( void )
+{
+  if( saved ) {
+    SDL_BlitSurface( saved, NULL, tmp_screen, NULL );
+    sdldisplay_force_full_refresh = 1;
+  }
 }
 
 static void
-sdl_blit_icon( SDL_Surface *icon, SDL_Rect *r, Uint32 tmp_screen_pitch,
+sdl_blit_icon( SDL_Surface **icon,
+               SDL_Rect *r, Uint32 tmp_screen_pitch,
                Uint32 dstPitch )
 {
-  int x = r->x;
-  int y = r->y;
-  int w = r->w;
-  int h = r->h;
-  int dst_y;
-  int dst_h;
+  int x, y, w, h, dst_y, dst_h;
 
+  if( timex ) {
+    r->x<<=1;
+    r->y<<=1;
+    r->w<<=1;
+    r->h<<=1;
+  }
+
+  x = r->x;
+  y = r->y;
+  w = r->w;
+  h = r->h;
   r->x++;
   r->y++;
 
-  if( SDL_BlitSurface( icon, NULL, tmp_screen, r ) ) return;
+  if( SDL_BlitSurface( icon[timex], NULL, tmp_screen, r ) ) return;
 
   /* Extend the dirty region by 1 pixel for scalers
      that "smear" the screen, e.g. 2xSAI */
@@ -362,13 +479,12 @@ sdl_blit_icon( SDL_Surface *icon, SDL_Rect *r, Uint32 tmp_screen_pitch,
 	dstPitch, w, dst_h
   );
 
-  /* Adjust rects for the destination rect size */
-
   if( num_rects == MAX_UPDATE_RECT ) {
     sdldisplay_force_full_refresh = 1;
     return;
   }
 
+  /* Adjust rects for the destination rect size */
   updated_rects[num_rects].x = x * sdldisplay_current_size;
   updated_rects[num_rects].y = dst_y;
   updated_rects[num_rects].w = w * sdldisplay_current_size;
@@ -380,7 +496,7 @@ sdl_blit_icon( SDL_Surface *icon, SDL_Rect *r, Uint32 tmp_screen_pitch,
 static void
 sdl_icon_overlay( Uint32 tmp_screen_pitch, Uint32 dstPitch )
 {
-  SDL_Rect r = { 264, 218, red_disk->w, red_disk->h };
+  SDL_Rect r = { 243, 218, red_disk[0]->w, red_disk[0]->h };
 
   switch( sdl_disk_state ) {
   case UI_STATUSBAR_STATE_ACTIVE:
@@ -393,20 +509,180 @@ sdl_icon_overlay( Uint32 tmp_screen_pitch, Uint32 dstPitch )
     break;
   }
 
+  r.x = 264;
+  r.y = 218;
+  r.w = red_mdr[0]->w;
+  r.h = red_mdr[0]->h;
+
+  switch( sdl_mdr_state ) {
+  case UI_STATUSBAR_STATE_ACTIVE:
+    sdl_blit_icon( green_mdr, &r, tmp_screen_pitch, dstPitch );
+    break;
+  case UI_STATUSBAR_STATE_INACTIVE:
+    sdl_blit_icon( red_mdr, &r, tmp_screen_pitch, dstPitch );
+    break;
+  case UI_STATUSBAR_STATE_NOT_AVAILABLE:
+    break;
+  }
+
   r.x = 285;
   r.y = 220;
-  r.w = red_cassette->w;
-  r.h = red_cassette->h;
+  r.w = red_cassette[0]->w;
+  r.h = red_cassette[0]->h;
 
   switch( sdl_tape_state ) {
   case UI_STATUSBAR_STATE_ACTIVE:
     sdl_blit_icon( green_cassette, &r, tmp_screen_pitch, dstPitch );
     break;
   case UI_STATUSBAR_STATE_INACTIVE:
+  case UI_STATUSBAR_STATE_NOT_AVAILABLE:
     sdl_blit_icon( red_cassette, &r, tmp_screen_pitch, dstPitch );
     break;
-  case UI_STATUSBAR_STATE_NOT_AVAILABLE:
-    break;
+  }
+
+  sdl_status_updated = 0;
+}
+
+/* Set one pixel in the display */
+void
+uidisplay_putpixel( int x, int y, int colour )
+{
+  libspectrum_word *dest_base, *dest;
+  Uint32 *palette_values = settings_current.bw_tv ? bw_values :
+                           colour_values;
+
+  Uint32 palette_colour = palette_values[ colour ];
+
+  if( machine_current->timex ) {
+    x <<= 1; y <<= 1;
+    dest_base = dest =
+      (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
+                           (x+1) * tmp_screen->format->BytesPerPixel +
+                           (y+1) * tmp_screen->pitch);
+
+    *(dest++) = palette_colour;
+    *(dest++) = palette_colour;
+    dest = (libspectrum_word*)
+      ( (libspectrum_byte*)dest_base + tmp_screen->pitch);
+    *(dest++) = palette_colour;
+    *(dest++) = palette_colour;
+  } else {
+    dest =
+      (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
+                           (x+1) * tmp_screen->format->BytesPerPixel +
+                           (y+1) * tmp_screen->pitch);
+
+    *dest = palette_colour;
+  }
+}
+
+/* Print the 8 pixels in `data' using ink colour `ink' and paper
+   colour `paper' to the screen at ( (8*x) , y ) */
+void
+uidisplay_plot8( int x, int y, libspectrum_byte data,
+	         libspectrum_byte ink, libspectrum_byte paper )
+{
+  libspectrum_word *dest;
+  Uint32 *palette_values = settings_current.bw_tv ? bw_values :
+                           colour_values;
+
+  Uint32 palette_ink = palette_values[ ink ];
+  Uint32 palette_paper = palette_values[ paper ];
+
+  if( machine_current->timex ) {
+    int i;
+    libspectrum_word *dest_base;
+
+    x <<= 4; y <<= 1;
+
+    dest_base =
+      (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
+                           (x+1) * tmp_screen->format->BytesPerPixel +
+                           (y+1) * tmp_screen->pitch);
+
+    for( i=0; i<2; i++ ) {
+      dest = dest_base;
+
+      *(dest++) = ( data & 0x80 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x80 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x40 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x40 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x20 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x20 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x10 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x10 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x08 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x08 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x04 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x04 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x02 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x02 ) ? palette_ink : palette_paper;
+      *(dest++) = ( data & 0x01 ) ? palette_ink : palette_paper;
+      *dest     = ( data & 0x01 ) ? palette_ink : palette_paper;
+
+      dest_base = (libspectrum_word*)
+        ( (libspectrum_byte*)dest_base + tmp_screen->pitch);
+    }
+  } else {
+    x <<= 3;
+
+    dest =
+      (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
+                           (x+1) * tmp_screen->format->BytesPerPixel +
+                           (y+1) * tmp_screen->pitch);
+
+    *(dest++) = ( data & 0x80 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x40 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x20 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x10 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x08 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x04 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x02 ) ? palette_ink : palette_paper;
+    *dest     = ( data & 0x01 ) ? palette_ink : palette_paper;
+  }
+}
+
+/* Print the 16 pixels in `data' using ink colour `ink' and paper
+   colour `paper' to the screen at ( (16*x) , y ) */
+void
+uidisplay_plot16( int x, int y, libspectrum_word data,
+		  libspectrum_byte ink, libspectrum_byte paper )
+{
+  libspectrum_word *dest_base, *dest;
+  int i;
+  Uint32 *palette_values = settings_current.bw_tv ? bw_values :
+                           colour_values;
+  Uint32 palette_ink = palette_values[ ink ];
+  Uint32 palette_paper = palette_values[ paper ];
+  x <<= 4; y <<= 1;
+
+  dest_base =
+    (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
+                         (x+1) * tmp_screen->format->BytesPerPixel +
+                         (y+1) * tmp_screen->pitch);
+
+  for( i=0; i<2; i++ ) {
+    dest = dest_base;
+
+    *(dest++) = ( data & 0x8000 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x4000 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x2000 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x1000 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0800 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0400 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0200 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0100 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0080 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0040 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0020 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0010 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0008 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0004 ) ? palette_ink : palette_paper;
+    *(dest++) = ( data & 0x0002 ) ? palette_ink : palette_paper;
+    *dest     = ( data & 0x0001 ) ? palette_ink : palette_paper;
+
+    dest_base = (libspectrum_word*)
+      ( (libspectrum_byte*)dest_base + tmp_screen->pitch);
   }
 }
 
@@ -416,15 +692,14 @@ uidisplay_frame_end( void )
   SDL_Rect *r;
   Uint32 tmp_screen_pitch, dstPitch;
   SDL_Rect *last_rect;
-  Uint32 *palette_values = settings_current.bw_tv ? bw_values :
-                           colour_values;
 
   /* We check for a switch to fullscreen here to give systems with a
      windowed-only UI a chance to free menu etc. resources before
      the switch to fullscreen (e.g. Mac OS X) */
-  if( sdldisplay_is_full_screen != settings_current.full_screen ) {
-    uidisplay_hotswap_gfx_mode();
-    sdldisplay_is_full_screen = !sdldisplay_is_full_screen;
+  if( sdldisplay_is_full_screen != settings_current.full_screen &&
+      uidisplay_hotswap_gfx_mode() ) {
+    fprintf( stderr, "%s: Error switching to fullscreen\n", fuse_progname );
+    fuse_abort();
   }
 
   /* Force a full redraw if requested */
@@ -438,12 +713,11 @@ uidisplay_frame_end( void )
   }
 
 #ifdef USE_WIDGET
-  if ( !(widget_level >= 0) && num_rects == 0 ) return;
+  if ( !(widget_level >= 0) && num_rects == 0 && !sdl_status_updated ) return;
 #else                   /* #ifdef USE_WIDGET */
-  if ( num_rects == 0 ) return;
+  if ( num_rects == 0 && !sdl_status_updated ) return;
 #endif                  /* #ifdef USE_WIDGET */
 
-  if( SDL_MUSTLOCK( tmp_screen ) ) SDL_LockSurface( tmp_screen );
   if( SDL_MUSTLOCK( sdldisplay_gc ) ) SDL_LockSurface( sdldisplay_gc );
 
   tmp_screen_pitch = tmp_screen->pitch;
@@ -454,25 +728,9 @@ uidisplay_frame_end( void )
 
   for( r = updated_rects; r != last_rect; r++ ) {
 
-    libspectrum_word *dest_base, *dest;
-    size_t xx,yy;
     int dst_y = r->y * sdldisplay_current_size;
     int dst_h = r->h;
 
-    dest_base =
-      (libspectrum_word*)( (libspectrum_byte*)tmp_screen->pixels +
-                           (r->x+1) * tmp_screen->format->BytesPerPixel +
-			   (r->y+1)*tmp_screen_pitch );
-
-    for( yy = r->y; yy < r->y + r->h; yy++ ) {
-
-      for( xx = r->x, dest = dest_base; xx < r->x + r->w; xx++, dest++ )
-        *dest = palette_values[ display_image[yy][xx] ];
-
-      dest_base = (libspectrum_word*)
-        ( (libspectrum_byte*)dest_base + tmp_screen_pitch );
-    }
-	  
     scaler_proc16(
       (libspectrum_byte*)tmp_screen->pixels +
                         (r->x+1) * tmp_screen->format->BytesPerPixel +
@@ -496,7 +754,6 @@ uidisplay_frame_end( void )
   if ( settings_current.statusbar )
     sdl_icon_overlay( tmp_screen_pitch, dstPitch );
 
-  if( SDL_MUSTLOCK( tmp_screen ) ) SDL_UnlockSurface( tmp_screen );
   if( SDL_MUSTLOCK( sdldisplay_gc ) ) SDL_UnlockSurface( sdldisplay_gc );
 
   /* Finally, blit all our changes to the screen */
@@ -533,23 +790,40 @@ uidisplay_area( int x, int y, int width, int height )
 int
 uidisplay_end( void )
 {
+  int i;
+
   display_ui_initialised = 0;
+
   if ( tmp_screen ) {
     free( tmp_screen->pixels );
     SDL_FreeSurface( tmp_screen ); tmp_screen = NULL;
   }
-  if ( red_cassette ) {
-    SDL_FreeSurface( red_cassette ); red_cassette = NULL;
+
+  if( saved ) {
+    SDL_FreeSurface( saved ); saved = NULL;
   }
-  if ( green_cassette ) {
-    SDL_FreeSurface( green_cassette ); green_cassette = NULL;
-  }
-  if ( red_disk ) {
-    SDL_FreeSurface( red_disk ); red_disk = NULL;
-  }
-  if ( green_disk ) {
-    SDL_FreeSurface( green_disk ); green_disk = NULL;
+
+  for( i=0; i<2; i++ ) {
+    if ( red_cassette[i] ) {
+      SDL_FreeSurface( red_cassette[i] ); red_cassette[i] = NULL;
+    }
+    if ( green_cassette[i] ) {
+      SDL_FreeSurface( green_cassette[i] ); green_cassette[i] = NULL;
+    }
+    if ( red_mdr[i] ) {
+      SDL_FreeSurface( red_mdr[i] ); red_mdr[i] = NULL;
+    }
+    if ( green_mdr[i] ) {
+      SDL_FreeSurface( green_mdr[i] ); green_mdr[i] = NULL;
+    }
+    if ( red_disk[i] ) {
+      SDL_FreeSurface( red_disk[i] ); red_disk[i] = NULL;
+    }
+    if ( green_disk[i] ) {
+      SDL_FreeSurface( green_disk[i] ); green_disk[i] = NULL;
+    }
   }
+
   return 0;
 }
 
@@ -561,14 +835,25 @@ ui_statusbar_update( ui_statusbar_item item, ui_statusbar_state state )
 
   case UI_STATUSBAR_ITEM_DISK:
     sdl_disk_state = state;
+    sdl_status_updated = 1;
     return 0;
 
   case UI_STATUSBAR_ITEM_PAUSED:
-    // We don't support pausing this version of Fuse
+    /* We don't support pausing this version of Fuse */
     return 0;
 
   case UI_STATUSBAR_ITEM_TAPE:
     sdl_tape_state = state;
+    sdl_status_updated = 1;
+    return 0;
+
+  case UI_STATUSBAR_ITEM_MICRODRIVE:
+    sdl_mdr_state = state;
+    sdl_status_updated = 1;
+    return 0;
+
+  case UI_STATUSBAR_ITEM_MOUSE:
+    /* We don't support showing a grab icon */
     return 0;
 
   }
@@ -577,5 +862,3 @@ ui_statusbar_update( ui_statusbar_item item, ui_statusbar_state state )
             item );
   return 1;
 }
-
-#endif        /* #ifdef UI_SDL */
