@@ -1,7 +1,7 @@
 /* sdlui.c: Routines for dealing with the SDL user interface
    Copyright (c) 2000-2002 Philip Kendall, Matan Ziv-Av, Fredrick Meunier
 
-   $Id: sdlui.c,v 1.7 2004/01/01 23:19:35 pak21 Exp $
+   $Id: sdlui.c 3395 2007-12-03 17:06:29Z zubzero $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,21 +13,18 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
-#ifdef UI_SDL			/* Use this iff we're using SDL */
-
 #include <stdio.h>
 #include <SDL.h>
 
@@ -35,15 +32,17 @@
 #include "fuse.h"
 #include "ui/ui.h"
 #include "ui/uidisplay.h"
+#include "settings.h"
 #include "sdldisplay.h"
 #include "sdljoystick.h"
 #include "sdlkeyboard.h"
 #include "ui/scaler/scaler.h"
+#include "menu.h"
 
 void
 atexit_proc( void )
 { 
-  SDL_ShowCursor(SDL_ENABLE);
+  SDL_ShowCursor( SDL_ENABLE );
   SDL_Quit();
 }
 
@@ -60,6 +59,14 @@ ui_init( int *argc, char ***argv )
   if ( error )
     return error;
 
+#ifndef __MORPHOS__
+  SDL_EnableUNICODE( 1 );
+#endif				/* #ifndef __MORPHOS__ */
+
+  sdlkeyboard_init();
+
+  ui_mouse_present = 1;
+
   return 0;
 }
 
@@ -77,6 +84,20 @@ ui_event( void )
       sdlkeyboard_keyrelease( &(event.key) );
       break;
 
+    case SDL_MOUSEBUTTONDOWN:
+      ui_mouse_button( event.button.button, 1 );
+      break;
+    case SDL_MOUSEBUTTONUP:
+      ui_mouse_button( event.button.button, 0 );
+      break;
+    case SDL_MOUSEMOTION:
+      if( ui_mouse_grabbed ) {
+        ui_mouse_motion( event.motion.x - 128, event.motion.y - 128 );
+        if( event.motion.x != 128 || event.motion.y != 128 )
+          SDL_WarpMouse( 128, 128 );
+      }	
+      break;
+
 #if defined USE_JOYSTICK && !defined HAVE_JSW_H
 
     case SDL_JOYBUTTONDOWN:
@@ -92,11 +113,22 @@ ui_event( void )
 #endif			/* if defined USE_JOYSTICK && !defined HAVE_JSW_H */
 
     case SDL_QUIT:
+#ifdef USE_WIDGET
+      fuse_emulation_pause();
+      menu_file_exit(0);
+      fuse_emulation_unpause();
+#else 				/* #ifdef USE_WIDGET */
       fuse_exiting = 1;
+#endif				/* #ifdef USE_WIDGET */
       break;
     case SDL_VIDEOEXPOSE:
       display_refresh_all();
       break;
+    case SDL_ACTIVEEVENT:
+      if( event.active.state & SDL_APPINPUTFOCUS ) {
+	if( event.active.gain ) ui_mouse_resume(); else ui_mouse_suspend();
+      }
+      break;
     default:
       break;
     }
@@ -114,6 +146,8 @@ ui_end( void )
   if ( error )
     return error;
 
+  sdlkeyboard_end();
+
   SDL_Quit();
 
   return 0;
@@ -123,12 +157,43 @@ int
 ui_statusbar_update_speed( float speed )
 {
   char buffer[15];
+  const char fuse[] = "Fuse";
 
-  snprintf( buffer, 15, "Fuse - %3.0f%%", speed );
+  snprintf( buffer, 15, "%s - %3.0f%%", fuse, speed );
 
-  SDL_WM_SetCaption( buffer, buffer );
+  /* FIXME: Icon caption should be snapshot name? */
+  SDL_WM_SetCaption( buffer, fuse );
 
   return 0;
 }
 
-#endif				/* #ifdef UI_SDL */
+int
+ui_mouse_grab( int startup )
+{
+  if( settings_current.full_screen ) {
+    SDL_WarpMouse( 128, 128 );
+    return 1;
+  }
+  if( startup ) return 0;
+
+  switch( SDL_WM_GrabInput( SDL_GRAB_ON ) ) {
+  case SDL_GRAB_ON:
+  case SDL_GRAB_FULLSCREEN:
+    SDL_ShowCursor( SDL_DISABLE );
+    SDL_WarpMouse( 128, 128 );
+    return 1;
+  default:
+    ui_error( UI_ERROR_WARNING, "Mouse grab failed" );
+    return 0;
+  }
+}
+
+int
+ui_mouse_release( int suspend )
+{
+  if( settings_current.full_screen ) return !suspend;
+
+  SDL_WM_GrabInput( SDL_GRAB_OFF );
+  SDL_ShowCursor( SDL_ENABLE );
+  return 0;
+}
