@@ -1,7 +1,7 @@
 /* svgajoystick.c: Joystick emulation (using svgalib)
    Copyright (c) 2003-4 Darren Salt
 
-   $Id: svgajoystick.c,v 1.4 2004/07/15 02:31:18 pak21 Exp $
+   $Id: svgajoystick.c 3749 2008-08-15 12:47:44Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
    Darren: linux@youmustbejoking.demon.co.uk
 
@@ -28,8 +27,6 @@
 
 #include <config.h>
 
-#ifdef UI_SVGA			/* Use this iff we're using svgalib */
-
 #if !defined USE_JOYSTICK || defined HAVE_JSW_H
 
 /* Fake joystick, or override UI-specific handling */
@@ -100,8 +97,7 @@ ui_joystick_end( void )
 }
 
 static void
-do_axis( int which, int position, input_joystick_button negative,
-	 input_joystick_button positive )
+do_axis( int which, int position, input_key negative, input_key positive )
 {
   input_event_t event1, event2;
 
@@ -150,5 +146,3 @@ ui_joystick_poll( void )
 }
 
 #endif			/* #if !defined USE_JOYSTICK || defined HAVE_JSW_H */
-
-#endif			/* #ifdef UI_SVGA */
