@@ -1,7 +1,7 @@
 /* svgaui.c: Routines for dealing with the svgalib user interface
    Copyright (c) 2000-2003 Philip Kendall, Matan Ziv-Av, Russell Marks
 
-   $Id: svgaui.c,v 1.14 2003/04/02 12:44:24 pak21 Exp $
+   $Id: svgaui.c 3096 2007-08-06 09:20:34Z pak21 $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,24 +13,23 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
-#ifdef UI_SVGA			/* Use this iff we're using svgalib */
-
 #include <stdio.h>
 
+#include <vga.h>
 #include <vgakeyboard.h>
+#include <vgamouse.h>
 
 #include "display.h"
 #include "fuse.h"
@@ -39,6 +38,8 @@
 #include "ui/ui.h"
 #include "ui/uidisplay.h"
 
+static int oldbutton = 0, oldx = 0, oldy = 0;
+
 int ui_init(int *argc, char ***argv)
 {
   int error;
@@ -49,13 +50,41 @@ int ui_init(int *argc, char ***argv)
   error = svgakeyboard_init();
   if(error) return error;
 
+  vga_setmousesupport( 1 );
+  mouse_setxrange( 0, 255);
+  mouse_setyrange( 0, 255);
+  mouse_setscale( 64 );
+  mouse_setwrap( MOUSE_WRAPX | MOUSE_WRAPY );
+  mouse_setposition( 128, 128 );
+
+  ui_mouse_present = 1;
+
   return 0;
 }
 
 int
 ui_event( void )
 {
+  int x, y, b, bd;
+
   keyboard_update();
+  mouse_update();
+
+  x = mouse_getx();
+  y = mouse_gety();
+  b = mouse_getbutton();
+
+  bd = b ^ oldbutton;
+  if( bd & MOUSE_LEFTBUTTON ) ui_mouse_button( 1, b & MOUSE_LEFTBUTTON );
+  if( bd & MOUSE_MIDDLEBUTTON ) ui_mouse_button( 2, b & MOUSE_MIDDLEBUTTON );
+  if( bd & MOUSE_RIGHTBUTTON ) ui_mouse_button( 3, b & MOUSE_RIGHTBUTTON );
+  oldbutton = b;
+
+  if( x != oldx || y != oldy ) {
+    ui_mouse_motion( x - oldx, y - oldy );
+    oldx = x; oldy = y;
+  }
+
   return 0;
 }
 
@@ -72,4 +101,14 @@ int ui_end(void)
   return 0;
 }
 
-#endif				/* #ifdef UI_SVGA */
+int
+ui_mouse_grab( int startup GCC_UNUSED )
+{
+  return 1;
+}
+
+int
+ui_mouse_release( int suspend )
+{
+  return !suspend;
+}
