@@ -1,7 +1,7 @@
 /* uijoystick.c: Joystick emulation (using libjsw)
    Copyright (c) 2003-2004 Darren Salt, Philip Kendall
 
-   $Id: uijoystick.c,v 1.11 2004/06/18 21:53:22 pak21 Exp $
+   $Id: uijoystick.c 3749 2008-08-15 12:47:44Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,14 +13,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
    Darren: linux@youmustbejoking.demon.co.uk
 
@@ -63,9 +62,8 @@
 static js_data_struct jsd[2];
 
 static void poll_joystick( int which );
-static void do_axis( int which, double position,
-		     input_joystick_button negative,
-		     input_joystick_button positive );
+static void do_axis( int which, double position, input_key negative,
+		     input_key positive );
 
 static int
 init_stick( int which, const char *const device,
@@ -147,7 +145,7 @@ ui_joystick_init( void )
   char *calibration;
   int error;
 
-  home = utils_get_home_path(); if( !home ) return 1;
+  home = compat_get_home_path(); if( !home ) return 1;
 
   /* Default calibration file is ~/.joystick */
   calibration = malloc( strlen( home ) + strlen( JSDefaultCalibration ) + 2 );
@@ -225,8 +223,7 @@ poll_joystick( int which )
 }
 
 static void
-do_axis( int which, double position, input_joystick_button negative,
-	 input_joystick_button positive )
+do_axis( int which, double position, input_key negative, input_key positive )
 {
   input_event_t event1, event2;
 
