@@ -1,7 +1,7 @@
 /* utils.c: some useful helper functions
-   Copyright (c) 1999-2004 Philip Kendall
+   Copyright (c) 1999-2008 Philip Kendall
 
-   $Id: utils.c,v 1.40.2.2 2004/02/17 00:09:01 pak21 Exp $
+   $Id: utils.c 3846 2008-11-20 07:17:21Z fredm $
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,49 +13,45 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Author contact information:
 
-   E-mail: pak21-fuse@srcf.ucam.org
-   Postal address: 15 Crescent Road, Wokingham, Berks, RG40 2DB, England
+   E-mail: philip-fuse@shadowmagic.org.uk
 
 */
 
 #include <config.h>
 
 #include <errno.h>
-#include <fcntl.h>
+#ifdef HAVE_LIBGEN_H
 #include <libgen.h>
-#include <limits.h>
-#include <stdio.h>
-#include <stdlib.h>
+#endif				/* #ifdef HAVE_LIBGEN_H */
 #include <string.h>
 #include <sys/stat.h>
-#include <sys/types.h>
-
-#ifdef HAVE_MMAP
-#include <sys/mman.h>
-#endif
-
-#include <sys/stat.h>
-#include <sys/types.h>
 #include <ui/ui.h>
 #include <unistd.h>
 
 #include <libspectrum.h>
 
 #include "dck.h"
+#include "disk/beta.h"
+#include "divide.h"
 #include "fuse.h"
+#include "if1.h"
+#include "if2.h"
+#include "machines/specplus3.h"
 #include "memory.h"
 #include "rzx.h"
+#include "settings.h"
+#include "simpleide.h"
 #include "snapshot.h"
-#include "specplus3.h"
 #include "tape.h"
-#include "trdos.h"
 #include "utils.h"
+#include "zxatasp.h"
+#include "zxcf.h"
 
 typedef struct path_context {
 
@@ -114,56 +110,82 @@ utils_open_file( const char *filename, int autoload,
     break;
 
   case LIBSPECTRUM_CLASS_DISK_PLUS3:
-#ifdef HAVE_765_H
-    error = machine_select( LIBSPECTRUM_MACHINE_PLUS3 ); if( error ) break;
-    error = specplus3_disk_insert( SPECPLUS3_DRIVE_A, filename );
-    if( autoload ) {
-
-      int fd; utils_file snap;
-
-      fd = utils_find_auxiliary_file( "disk_plus3.z80", UTILS_AUXILIARY_LIB );
-      if( fd == -1 ) {
-	ui_error( UI_ERROR_ERROR, "Couldn't find +3 disk autoload snap" );
-	error = 1;
-	goto escape;
-      }
-
-      error = utils_read_fd( fd, "disk_plus3.z80", &snap );
-      if( error ) { utils_close_file( &snap ); goto escape; }
-
-      error = snapshot_read_buffer( snap.buffer, snap.length,
-				    LIBSPECTRUM_ID_SNAPSHOT_Z80 );
-      if( error ) { utils_close_file( &snap ); goto escape; }
-
-      if( utils_close_file( &snap ) ) {
-	ui_error( UI_ERROR_ERROR, "Couldn't munmap 'disk_plus3.z80': %s",
-		  strerror( errno ) );
-	error = 1;
-	goto escape;
-      }
+    if( !( machine_current->capabilities &
+	   LIBSPECTRUM_MACHINE_CAPABILITY_PLUS3_DISK ) ) {
+      error = machine_select( LIBSPECTRUM_MACHINE_PLUS3 ); if( error ) break;
     }
-  escape:
+
+    error = specplus3_disk_insert( SPECPLUS3_DRIVE_A, filename, autoload );
     break;
 
-#else				/* #ifdef HAVE_765_H */
-    ui_error( UI_ERROR_INFO, "lib765 not present so can't handle .dsk files" );
-#endif				/* #ifdef HAVE_765_H */
+  case LIBSPECTRUM_CLASS_DISK_PLUSD:
+
+    error = plusd_disk_insert( PLUSD_DRIVE_1, filename, autoload );
     break;
 
   case LIBSPECTRUM_CLASS_DISK_TRDOS:
-    error = machine_select( LIBSPECTRUM_MACHINE_PENT ); if( error ) break;
-    error = trdos_disk_insert( TRDOS_DRIVE_A, filename ); if( error ) break;
-    if( autoload ) {
-      machine_current->ram.current_rom = 1;
-      trdos_active = 1;
-      memory_map[0].page = &ROM[2][0x0000];
-      memory_map[1].page = &ROM[2][0x2000];
+
+    if( !( machine_current->capabilities &
+	   LIBSPECTRUM_MACHINE_CAPABILITY_TRDOS_DISK ) &&
+        !periph_beta128_active ) {
+      error = machine_select( LIBSPECTRUM_MACHINE_PENT ); if( error ) break;
     }
+
+    error = beta_disk_insert( BETA_DRIVE_A, filename, autoload );
+    break;
+
+  case LIBSPECTRUM_CLASS_DISK_GENERIC:
+    if( machine_current->machine == LIBSPECTRUM_MACHINE_PLUS3 ||
+        machine_current->machine == LIBSPECTRUM_MACHINE_PLUS2A )
+      error = specplus3_disk_insert( SPECPLUS3_DRIVE_A, filename, autoload );
+    else if( machine_current->machine == LIBSPECTRUM_MACHINE_PENT ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_PENT512 ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_PENT1024 ||
+          machine_current->machine == LIBSPECTRUM_MACHINE_SCORP )
+      error = beta_disk_insert( BETA_DRIVE_A, filename, autoload );
+    else
+      if( periph_beta128_active )
+        error = beta_disk_insert( BETA_DRIVE_A, filename, autoload );
+      else if( periph_plusd_active )
+        error = plusd_disk_insert( PLUSD_DRIVE_1, filename, autoload );
+    break;
+
+  case LIBSPECTRUM_CLASS_CARTRIDGE_IF2:
+    error = if2_insert( filename );
+    break;
+
+  case LIBSPECTRUM_CLASS_MICRODRIVE:
+    error = if1_mdr_insert( -1, filename );
     break;
 
   case LIBSPECTRUM_CLASS_CARTRIDGE_TIMEX:
-    error = machine_select( LIBSPECTRUM_MACHINE_TC2068 ); if( error ) break;
-    error = dck_read( filename );
+    if( !( machine_current->capabilities &
+	   LIBSPECTRUM_MACHINE_CAPABILITY_TIMEX_DOCK ) ) {
+      error = machine_select( LIBSPECTRUM_MACHINE_TC2068 ); if( error ) break;
+    }
+    error = dck_insert( filename );
+    break;
+
+  case LIBSPECTRUM_CLASS_HARDDISK:
+    if( !settings_current.simpleide_active &&
+	!settings_current.zxatasp_active   &&
+	!settings_current.divide_enabled   &&
+	!settings_current.zxcf_active         ) {
+      settings_current.zxcf_active = 1;
+      periph_update();
+    }
+
+    if( settings_current.zxcf_active ) {
+      error = zxcf_insert( filename );
+    } else if( settings_current.zxatasp_active ) {
+      error = zxatasp_insert( filename, LIBSPECTRUM_IDE_MASTER );
+    } else if( settings_current.simpleide_active ) {
+      error = simpleide_insert( filename, LIBSPECTRUM_IDE_MASTER );
+    } else {
+      error = divide_insert( filename, LIBSPECTRUM_IDE_MASTER );
+    }
+    if( error ) return error;
+    
     break;
 
   default:
@@ -181,27 +203,31 @@ utils_open_file( const char *filename, int autoload,
   return 0;
 }
 
-/* Find the auxillary file called `filename'; returns a fd for the
+/* Find the auxiliary file called `filename'; returns a fd for the
    file on success, -1 if it couldn't find the file */
 int
 utils_find_auxiliary_file( const char *filename, utils_aux_type type )
 {
-  int fd;
+  compat_fd fd;
 
   char path[ PATH_MAX ];
   path_context ctx;
 
   /* If given an absolute path, just look there */
-  if( filename[0] == '/' ) return open( filename, O_RDONLY | O_BINARY );
+  if( compat_is_absolute_path( filename ) )
+    return compat_file_open( filename, 0 );
 
   /* Otherwise look in some likely locations */
   init_path_context( &ctx, type );
 
   while( get_next_path( &ctx ) ) {
-
-    snprintf( path, PATH_MAX, "%s/%s", ctx.path, filename );
-    fd = open( path, O_RDONLY | O_BINARY );
-    if( fd != -1 ) return fd;
+#ifdef AMIGA
+    snprintf( path, PATH_MAX, "%s%s", ctx.path, filename );
+#else
+    snprintf( path, PATH_MAX, "%s" FUSE_DIR_SEP_STR "%s", ctx.path, filename );
+#endif
+    fd = compat_file_open( path, 0 );
+    if( fd != COMPAT_FILE_OPEN_FAILED ) return fd;
 
   }
 
@@ -218,7 +244,7 @@ utils_find_file_path( const char *filename, char *ret_path,
   struct stat stat_info;
 
   /* If given an absolute path, just look there */
-  if( filename[0] == '/' ) {
+  if( compat_is_absolute_path( filename ) ) {
     strncpy( ret_path, filename, PATH_MAX );
     return 0;
   }
@@ -228,7 +254,12 @@ utils_find_file_path( const char *filename, char *ret_path,
 
   while( get_next_path( &ctx ) ) {
 
-    snprintf( ret_path, PATH_MAX, "%s/%s", ctx.path, filename );
+#ifdef AMIGA
+    snprintf( ret_path, PATH_MAX, "%s%s", ctx.path, filename );
+#else
+    snprintf( ret_path, PATH_MAX, "%s" FUSE_DIR_SEP_STR "%s", ctx.path,
+              filename );
+#endif
     if( !stat( ret_path, &stat_info ) ) return 0;
 
   }
@@ -255,28 +286,41 @@ get_next_path( path_context *ctx )
   switch( (ctx->state)++ ) {
 
     /* First look relative to the current directory */
-  case 0: strncpy( ctx->path, ".", PATH_MAX ); return 1;
+  case 0:
+#ifdef AMIGA
+    strncpy( ctx->path, "PROGDIR:", PATH_MAX );
+#else
+    strncpy( ctx->path, ".", PATH_MAX );
+#endif
+    return 1;
 
     /* Then relative to the Fuse executable */
   case 1:
 
     switch( ctx->type ) {
-    
+#ifdef AMIGA
+    case UTILS_AUXILIARY_LIB: strncpy( ctx->path, "PROGDIR:lib/", PATH_MAX); return 1;
+    case UTILS_AUXILIARY_ROM: strncpy( ctx->path, "PROGDIR:roms/", PATH_MAX); return 1;
+    case UTILS_AUXILIARY_WIDGET: strncpy( ctx->path, "PROGDIR:ui/widget/", PATH_MAX); return 1;
+#else
     case UTILS_AUXILIARY_LIB: path_segment = "lib"; break;
     case UTILS_AUXILIARY_ROM: path_segment = "roms"; break;
+    case UTILS_AUXILIARY_WIDGET: path_segment = "ui/widget"; break;
+#endif
     default:
       ui_error( UI_ERROR_ERROR, "unknown auxiliary file type %d", ctx->type );
       return 0;
     }
 
-    if( fuse_progname[0] == '/' ) {
+    if( compat_is_absolute_path( fuse_progname ) ) {
       strncpy( buffer, fuse_progname, PATH_MAX );
       buffer[ PATH_MAX - 1 ] = '\0';
     } else {
       snprintf( buffer, PATH_MAX, "%s%s", fuse_directory, fuse_progname );
     }
     path2 = dirname( buffer );
-    snprintf( ctx->path, PATH_MAX, "%s/%s", path2, path_segment );
+    snprintf( ctx->path, PATH_MAX, "%s" FUSE_DIR_SEP_STR "%s", path2,
+              path_segment );
     return 1;
 
     /* Then where we may have installed the data files */
@@ -292,21 +336,19 @@ get_next_path( path_context *ctx )
 
   case 3: return 0;
   }
-
   ui_error( UI_ERROR_ERROR, "unknown path_context state %d", ctx->state );
   fuse_abort();
-  return 1;			/* Keep gcc happy */
 }
 
 int
 utils_read_file( const char *filename, utils_file *file )
 {
-  int fd;
+  compat_fd fd;
 
   int error;
 
-  fd = open( filename, O_RDONLY | O_BINARY );
-  if( fd == -1 ) {
+  fd = compat_file_open( filename, 0 );
+  if( fd == COMPAT_FILE_OPEN_FAILED ) {
     ui_error( UI_ERROR_ERROR, "couldn't open '%s': %s", filename,
 	      strerror( errno ) );
     return 1;
@@ -319,58 +361,24 @@ utils_read_file( const char *filename, utils_file *file )
 }
 
 int
-utils_read_fd( int fd, const char *filename, utils_file *file )
+utils_read_fd( compat_fd fd, const char *filename, utils_file *file )
 {
-  struct stat file_info;
-
-  if( fstat( fd, &file_info) ) {
-    ui_error( UI_ERROR_ERROR, "Couldn't stat '%s': %s", filename,
-	      strerror( errno ) );
-    close(fd);
-    return 1;
-  }
-
-  file->length = file_info.st_size;
-
-#ifdef HAVE_MMAP
-
-  file->buffer = mmap( 0, file->length, PROT_READ, MAP_SHARED, fd, 0 );
-
-  if( file->buffer != (void*)-1 ) {
-
-    file->mode = UTILS_FILE_OPEN_MMAP;
-
-    if( close(fd) ) {
-      ui_error( UI_ERROR_ERROR, "Couldn't close '%s': %s", filename,
-		strerror( errno ) );
-      munmap( file->buffer, file->length );
-      return 1;
-    }
-
-    return 0;
-  }
-
-#endif			/* #ifdef HAVE_MMAP */
-
-  /* Either mmap() isn't available, or it failed for some reason so try
-     using normal IO */
-  file->mode = UTILS_FILE_OPEN_MALLOC;
+  file->length = compat_file_get_length( fd );
+  if( file->length == -1 ) return 1;
 
   file->buffer = malloc( file->length );
   if( !file->buffer ) {
-    ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d\n", __FILE__, __LINE__ );
+    ui_error( UI_ERROR_ERROR, "Out of memory at %s:%d", __FILE__, __LINE__ );
     return 1;
   }
 
-  if( read( fd, file->buffer, file->length ) != file->length ) {
-    ui_error( UI_ERROR_ERROR, "Error reading from '%s': %s", filename,
-	      strerror( errno ) );
+  if( compat_file_read( fd, file ) ) {
     free( file->buffer );
-    close( fd );
+    compat_file_close( fd );
     return 1;
   }
 
-  if( close(fd) ) {
+  if( compat_file_close( fd ) ) {
     ui_error( UI_ERROR_ERROR, "Couldn't close '%s': %s", filename,
 	      strerror( errno ) );
     free( file->buffer );
@@ -383,31 +391,7 @@ utils_read_fd( int fd, const char *filename, utils_file *file )
 int
 utils_close_file( utils_file *file )
 {
-  switch( file->mode ) {
-
-  case UTILS_FILE_OPEN_MMAP:
-#ifdef HAVE_MMAP
-    if( file->length ) {
-      if( munmap( file->buffer, file->length ) ) {
-	ui_error( UI_ERROR_ERROR, "Couldn't munmap: %s\n", strerror( errno ) );
-	return 1;
-      }
-    }
-#else				/* #ifdef HAVE_MMAP */
-    ui_error( UI_ERROR_ERROR, "utils_close_file: file->mode == UTILS_FILE_OPEN_MMAP, but mmap not available?!" );
-    fuse_abort();
-#endif
-    break;
-
-  case UTILS_FILE_OPEN_MALLOC:
-    free( file->buffer );
-    break;
-
-  default:
-    ui_error( UI_ERROR_ERROR, "Unknown file open mode %d\n", file->mode );
-    fuse_abort();
-
-  }
+  free( file->buffer );
 
   return 0;
 }
@@ -415,28 +399,22 @@ utils_close_file( utils_file *file )
 int utils_write_file( const char *filename, const unsigned char *buffer,
 		      size_t length )
 {
-  FILE *f;
+  compat_fd fd;
 
-  f=fopen( filename, "wb" );
-  if(!f) { 
-    ui_error( UI_ERROR_ERROR, "error opening '%s': %s", filename,
-	      strerror( errno ) );
-    return 1;
-  }
-	    
-  if( fwrite( buffer, 1, length, f ) != length ) {
-    ui_error( UI_ERROR_ERROR, "error writing to '%s': %s", filename,
-	      strerror( errno ) );
-    fclose(f);
+  fd = compat_file_open( filename, 1 );
+  if( fd == COMPAT_FILE_OPEN_FAILED ) {
+    ui_error( UI_ERROR_ERROR, "couldn't open `%s' for writing: %s\n",
+    	      filename, strerror( errno ) );
     return 1;
   }
 
-  if( fclose( f ) ) {
-    ui_error( UI_ERROR_ERROR, "error closing '%s': %s", filename,
-	      strerror( errno ) );
+  if( compat_file_write( fd, buffer, length ) ) {
+    compat_file_close( fd );
     return 1;
   }
 
+  if( compat_file_close( fd ) ) return 1;
+
   return 0;
 }
 
@@ -449,12 +427,18 @@ utils_make_temp_file( int *fd, char *tempfilename, const char *filename,
   utils_file file;
   ssize_t bytes_written;
 
-  snprintf( tempfilename, PATH_MAX, "%s/%s", utils_get_temp_path(), template );
+#if defined AMIGA || defined __MORPHOS__
+  snprintf( tempfilename, PATH_MAX, "%s%s", compat_get_temp_path(), template );
+#else
+  snprintf( tempfilename, PATH_MAX, "%s" FUSE_DIR_SEP_STR "%s",
+            compat_get_temp_path(), template );
+#endif
 
   *fd = mkstemp( tempfilename );
   if( *fd == -1 ) {
     ui_error( UI_ERROR_ERROR, "couldn't create temporary file: %s",
 	      strerror( errno ) );
+    return 1;
   }
 
   error = utils_read_file( filename, &file );
@@ -480,49 +464,3 @@ utils_make_temp_file( int *fd, char *tempfilename, const char *filename,
 
   return 0;
 }
-
-/* Get a path where we can reasonably create temporary files */
-const char*
-utils_get_temp_path( void )
-{
-  const char *dir;
-
-#ifdef WIN32
-
-  /* Something close to this algorithm specified at
-     http://msdn.microsoft.com/library/default.asp?url=/library/en-us/fileio/base/gettemppath.asp
-  */
-  dir = getenv( "TMP" ); if( dir ) return dir;
-  dir = getenv( "TEMP" ); if( dir ) return dir;
-  return ".";
-
-#else				/* #ifdef WIN32 */
-
-  /* Unix-ish. Use TMPDIR if specified, if not /tmp */
-  dir = getenv( "TMPDIR" ); if( dir ) return dir;
-  return "/tmp";
-
-#endif				/* #ifdef WIN32 */
-  
-}
-
-/* Return the path where we will store our configuration information etc */
-const char*
-utils_get_home_path( void )
-{
-  const char *dir;
-
-#ifdef WIN32
-
-  dir = getenv( "USERPROFILE" ); if( dir ) return dir;
-  dir = getenv( "WINDIR" ); if( dir ) return dir;
-
-#else				/* #ifdef WIN32 */
-
-  dir = getenv( "HOME" ); if( dir ) return dir;
-
-#endif				/* #ifdef WIN32 */
-
-  ui_error( UI_ERROR_ERROR, "couldn't find a plausible home directory" );
-  return NULL;
-}
